package org.jeecg.modules.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.system.entity.FileInfo;
import org.jeecg.modules.system.service.IFileInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
* @Description: 文件信息表
* @Author: jeecg-boot
* @Date:   2023-02-21
* @Version: V1.0
*/
@Api(tags="文件信息表")
@RestController
@RequestMapping("/fileinfo/fileInfo")
@Slf4j
public class FileInfoController extends JeecgController<FileInfo, IFileInfoService> {
   @Autowired
   private IFileInfoService fileInfoService;

   /**
    * 分页列表查询
    *
    * @param fileInfo
    * @param pageNo
    * @param pageSize
    * @param req
    * @return
    */
   //@AutoLog(value = "文件信息表-分页列表查询")
   @ApiOperation(value="文件信息表-分页列表查询", notes="文件信息表-分页列表查询")
   @GetMapping(value = "/list")
   public Result<IPage<FileInfo>> queryPageList(FileInfo fileInfo,
                                  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
                                  HttpServletRequest req) {
       QueryWrapper<FileInfo> queryWrapper = QueryGenerator.initQueryWrapper(fileInfo, req.getParameterMap());
       Page<FileInfo> page = new Page<FileInfo>(pageNo, pageSize);
       IPage<FileInfo> pageList = fileInfoService.page(page, queryWrapper);
       return Result.OK(pageList);
   }

   /**
    *   添加
    *
    * @param fileInfo
    * @return
    */
   @AutoLog(value = "文件信息表-添加")
   @ApiOperation(value="文件信息表-添加", notes="文件信息表-添加")
   //@RequiresPermissions("fileinfo:file_info:add")
   @PostMapping(value = "/add")
   public Result<String> add(@RequestBody FileInfo fileInfo) {
       fileInfoService.save(fileInfo);
       return Result.OK("添加成功！");
   }

   /**
    *  编辑
    *
    * @param fileInfo
    * @return
    */
   @AutoLog(value = "文件信息表-编辑")
   @ApiOperation(value="文件信息表-编辑", notes="文件信息表-编辑")
   //@RequiresPermissions("fileinfo:file_info:edit")
   @RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
   public Result<String> edit(@RequestBody FileInfo fileInfo) {
       fileInfoService.updateById(fileInfo);
       return Result.OK("编辑成功!");
   }

   /**
    *   通过id删除
    *
    * @param id
    * @return
    */
   @AutoLog(value = "文件信息表-通过id删除")
   @ApiOperation(value="文件信息表-通过id删除", notes="文件信息表-通过id删除")
   //@RequiresPermissions("fileinfo:file_info:delete")
   @DeleteMapping(value = "/delete")
   public Result<String> delete(@RequestParam(name="id",required=true) String id) {
       fileInfoService.removeById(id);
       return Result.OK("删除成功!");
   }

   /**
    *  批量删除
    *
    * @param ids
    * @return
    */
   @AutoLog(value = "文件信息表-批量删除")
   @ApiOperation(value="文件信息表-批量删除", notes="文件信息表-批量删除")
   //@RequiresPermissions("fileinfo:file_info:deleteBatch")
   @DeleteMapping(value = "/deleteBatch")
   public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
       this.fileInfoService.removeByIds(Arrays.asList(ids.split(",")));
       return Result.OK("批量删除成功!");
   }

   /**
    * 通过id查询
    *
    * @param id
    * @return
    */
   //@AutoLog(value = "文件信息表-通过id查询")
   @ApiOperation(value="文件信息表-通过id查询", notes="文件信息表-通过id查询")
   @GetMapping(value = "/queryById")
   public Result<FileInfo> queryById(@RequestParam(name="id",required=true) String id) {
       FileInfo fileInfo = fileInfoService.getById(id);
       if(fileInfo==null) {
           return Result.error("未找到对应数据");
       }
       return Result.OK(fileInfo);
   }

   /**
   * 导出excel
   *
   * @param request
   * @param fileInfo
   */
   //@RequiresPermissions("fileinfo:file_info:exportXls")
   @RequestMapping(value = "/exportXls")
   public ModelAndView exportXls(HttpServletRequest request, FileInfo fileInfo) {
       return super.exportXls(request, fileInfo, FileInfo.class, "文件信息表");
   }

   /**
     * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
   //@RequiresPermissions("fileinfo:file_info:importExcel")
   @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
   public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
       return super.importExcel(request, response, FileInfo.class);
   }

    @ApiOperation(value = "根据文件id下载指定文件", notes = "根据文件id下载指定文件")
    @GetMapping(value = "/downloadByFileSaveId")
    public void downloadByFileSaveId(String fileSaveId, HttpServletResponse response) {
        fileInfoService.downloadFile(fileSaveId, response);
    }
}
