package com.ynfy.buss.course.coursecatalog.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.course.coursecatalog.entity.CourseCatalog;
import com.ynfy.buss.course.coursecatalog.service.ICourseCatalogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.system.entity.FileInfo;
import org.jeecg.modules.system.service.IFileInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: course_catalog
 * @Author: jeecg-boot
 * @Date: 2023-10-24
 * @Version: V1.0
 */
@Api(tags = "course_catalog")
@RestController
@RequestMapping("/courseCatalog")
@Slf4j
public class CourseCatalogController extends JeecgController<CourseCatalog, ICourseCatalogService> {
    @Autowired
    private ICourseCatalogService courseCatalogService;

    @Autowired
    private IFileInfoService fileInfoService;


    /**
     * 分页列表查询
     *
     * @param courseCatalog
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "course_catalog-分页列表查询")
    @ApiOperation(value = "course_catalog-分页列表查询", notes = "course_catalog-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<CourseCatalog>> queryPageList(CourseCatalog courseCatalog,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        QueryWrapper<CourseCatalog> queryWrapper = QueryGenerator.initQueryWrapper(courseCatalog, req.getParameterMap());
        Page<CourseCatalog> page = new Page<CourseCatalog>(pageNo, pageSize);
        IPage<CourseCatalog> pageList = courseCatalogService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param courseCatalog
     * @return
     */
    @AutoLog(value = "course_catalog-添加")
    @ApiOperation(value = "course_catalog-添加", notes = "course_catalog-添加")
    //@RequiresPermissions("courseCatalog:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody CourseCatalog courseCatalog) {
        courseCatalogService.save(courseCatalog);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param courseCatalog
     * @return
     */
    @AutoLog(value = "course_catalog-编辑")
    @ApiOperation(value = "course_catalog-编辑", notes = "course_catalog-编辑")
    //@RequiresPermissions("courseCatalog:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody CourseCatalog courseCatalog) {
        courseCatalogService.updateById(courseCatalog);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "course_catalog-通过id删除")
    @ApiOperation(value = "course_catalog-通过id删除", notes = "course_catalog-通过id删除")
    //@RequiresPermissions("courseCatalog:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        courseCatalogService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "course_catalog-批量删除")
    @ApiOperation(value = "course_catalog-批量删除", notes = "course_catalog-批量删除")
    //@RequiresPermissions("courseCatalog:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.courseCatalogService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "course_catalog-通过id查询")
    @ApiOperation(value = "course_catalog-通过id查询", notes = "course_catalog-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<CourseCatalog> queryById(@RequestParam(name = "id", required = true) String id) {
        CourseCatalog courseCatalog = courseCatalogService.getById(id);
        if (courseCatalog == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(courseCatalog);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param courseCatalog
     */
    //@RequiresPermissions("courseCatalog:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, CourseCatalog courseCatalog) {
        return super.exportXls(request, courseCatalog, CourseCatalog.class, "course_catalog");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("courseCatalog:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, CourseCatalog.class);
    }

    /**
     * 获取视频时长 秒
     *
     * @return
     */
    @GetMapping("/getVideoDuration")
    public Result<?> getVideoDuration(@RequestParam String fileSaveId) {
        FileInfo fileInfo = fileInfoService.listByFileSaveId(fileSaveId);
        return Result.OK("", fileInfo.getVideoDuration());
    }

    /**
     * 检查上一个任务是否完成
     *
     * @param id
     * @return
     */
    @GetMapping("/checkPreTaskComplete")
    public Result<?> checkPreTaskComplete(@RequestParam String id) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        return Result.OK(courseCatalogService.checkPreTaskComplete(id, user));
    }

}
