package com.ynfy.buss.course.usercoursecatalog.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.course.usercoursecatalog.entity.UserCourseCatalog;
import com.ynfy.buss.course.usercoursecatalog.entity.vo.CatalogTimeVO;
import com.ynfy.buss.course.usercoursecatalog.service.IUserCourseCatalogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: 用户课程任务学习情况
 * @Author: jeecg-boot
 * @Date: 2023-10-25
 * @Version: V1.0
 */
@Api(tags = "用户课程任务学习情况")
@RestController
@RequestMapping("/userCourseCatalog")
@Slf4j
public class UserCourseCatalogController extends JeecgController<UserCourseCatalog, IUserCourseCatalogService> {
    @Autowired
    private IUserCourseCatalogService userCourseCatalogService;

    /**
     * 分页列表查询
     *
     * @param userCourseCatalog
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "用户课程任务学习情况-分页列表查询")
    @ApiOperation(value = "用户课程任务学习情况-分页列表查询", notes = "用户课程任务学习情况-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserCourseCatalog>> queryPageList(UserCourseCatalog userCourseCatalog,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Page<UserCourseCatalog> page = new Page<>(pageNo, pageSize);
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        userCourseCatalog.setUserId(user.getId());
        IPage<UserCourseCatalog> pageList = userCourseCatalogService.selectPageList(page, userCourseCatalog);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param userCourseCatalog
     * @return
     */
    @AutoLog(value = "用户课程任务学习情况-添加")
    @ApiOperation(value = "用户课程任务学习情况-添加", notes = "用户课程任务学习情况-添加")
    @RequiresPermissions("userCourseCatalog:user_course_catalog:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody UserCourseCatalog userCourseCatalog) {
        userCourseCatalogService.save(userCourseCatalog);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param userCourseCatalog
     * @return
     */
    @AutoLog(value = "用户课程任务学习情况-编辑")
    @ApiOperation(value = "用户课程任务学习情况-编辑", notes = "用户课程任务学习情况-编辑")
    //@RequiresPermissions("userCourseCatalog:user_course_catalog:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody UserCourseCatalog userCourseCatalog) {
        userCourseCatalogService.updateById(userCourseCatalog);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户课程任务学习情况-通过id删除")
    @ApiOperation(value = "用户课程任务学习情况-通过id删除", notes = "用户课程任务学习情况-通过id删除")
    //@RequiresPermissions("userCourseCatalog:user_course_catalog:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        userCourseCatalogService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户课程任务学习情况-批量删除")
    @ApiOperation(value = "用户课程任务学习情况-批量删除", notes = "用户课程任务学习情况-批量删除")
    //@RequiresPermissions("userCourseCatalog:user_course_catalog:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.userCourseCatalogService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "用户课程任务学习情况-通过id查询")
    @ApiOperation(value = "用户课程任务学习情况-通过id查询", notes = "用户课程任务学习情况-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserCourseCatalog> queryById(@RequestParam(name = "id", required = true) String id) {
        UserCourseCatalog userCourseCatalog = userCourseCatalogService.getById(id);
        if (userCourseCatalog == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(userCourseCatalog);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param userCourseCatalog
     */
    //@RequiresPermissions("userCourseCatalog:user_course_catalog:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, UserCourseCatalog userCourseCatalog) {
        return super.exportXls(request, userCourseCatalog, UserCourseCatalog.class, "用户课程任务学习情况");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("userCourseCatalog:user_course_catalog:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, UserCourseCatalog.class);
    }

    /**
     * 计算任务学习时长
     *
     * @return
     */
    @AutoLog(value = "计算任务学习时长")
    @ApiOperation(value = "计算任务学习时长", notes = "计算任务学习时长")
    @PostMapping(value = "/calcStudyTime")
    public Result<String> calcStudyTime(@RequestBody CatalogTimeVO catalogTimeVO) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        userCourseCatalogService.calcStudyTime(catalogTimeVO, user.getId());
        return Result.OK();
    }


    /**
     * 获取课程任务学习进度
     *
     * @return
     */
    @AutoLog(value = "获取课程任务学习进度")
    @ApiOperation(value = "获取课程任务学习进度", notes = "获取课程任务学习进度")
    @GetMapping(value = "/getCourseStudyProcess")
    public Result<?> getCourseStudyProcess(@RequestParam String courseId) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        return Result.OK(userCourseCatalogService.getCourseStudyProcess(courseId, user.getId()));
    }

    /**
     * 获取任务学习记录
     *
     * @return
     */
    @AutoLog(value = "获取任务学习记录")
    @ApiOperation(value = "获取任务学习记录", notes = "获取任务学习记录")
    @GetMapping(value = "/findUserCatalog")
    public Result<?> findUserCatalog(@RequestParam String courseCatalogId) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        return Result.OK(userCourseCatalogService.findUserCatalog(user.getId(), courseCatalogId));
    }

}
