package com.ynfy.buss.exam.exam.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ynfy.buss.exam.exam.dto.*;
import com.ynfy.buss.exam.exam.entity.Exam;
import com.ynfy.buss.exam.exam.entity.ExamPapers;
import com.ynfy.buss.exam.exam.enums.ExamState;
import com.ynfy.buss.exam.exam.enums.PaperState;
import com.ynfy.buss.exam.exam.mapper.ExamMapper;
import com.ynfy.buss.exam.exam.service.IExamPapersService;
import com.ynfy.buss.exam.exam.service.IExamService;
import com.ynfy.buss.exam.job.BreakExamJob;
import com.ynfy.buss.exam.paper.enmus.JoinType;
import com.ynfy.buss.exam.paper.entity.Paper;
import com.ynfy.buss.exam.paper.service.IPaperService;
import com.ynfy.buss.exam.paperquestion.entity.PaperQuestion;
import com.ynfy.buss.exam.paperquestion.service.IPaperQuestionService;
import com.ynfy.buss.exam.paperrulegroup.entity.PaperRuleGroup;
import com.ynfy.buss.exam.paperrulegroup.service.IPaperRuleGroupService;
import com.ynfy.buss.exam.question.entity.Question;
import com.ynfy.buss.exam.question.enums.QuestionType;
import com.ynfy.buss.exam.question.service.IQuestionService;
import com.ynfy.buss.exam.questionanswer.entity.QuestionAnswer;
import com.ynfy.buss.exam.userexam.entity.UserExam;
import com.ynfy.buss.exam.userexam.entity.UserExamPapers;
import com.ynfy.buss.exam.userexam.service.BeanContext;
import com.ynfy.buss.exam.userexam.service.IUserExamPapersService;
import com.ynfy.buss.exam.userexam.service.IUserExamService;
import com.ynfy.buss.exam.userexamquestion.entity.UserExamQuestion;
import com.ynfy.buss.exam.userexamquestion.service.IUserExamQuestionService;
import com.ynfy.buss.exam.userexamquestionanswer.entity.UserExamQuestionAnswer;
import com.ynfy.buss.exam.userexamquestionanswer.service.IUserExamQuestionAnswerService;
import com.ynfy.buss.exam.userexamresult.service.IUserExamResultService;
import com.ynfy.common.utils.CharUtil;
import com.ynfy.common.utils.CronUtils;
import com.ynfy.common.utils.ThreadPoolUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.RedisUtil;
import org.jeecg.modules.quartz.entity.QuartzJob;
import org.jeecg.modules.quartz.service.IQuartzJobService;
import org.jeecg.modules.system.service.ISysDepartService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * @Description: exam
 * @Author: jeecg-boot
 * @Date: 2023-02-23
 * @Version: V1.0
 */
@Service
public class ExamServiceImpl extends ServiceImpl<ExamMapper, Exam> implements IExamService {

    @Autowired
    public RedisUtil redisUtil;
    @Autowired
    private ExamMapper examMapper;
    @Autowired
    private IPaperService paperService;
    @Autowired
    private IUserExamService userExamService;
    @Autowired
    private IPaperQuestionService paperQuestionService;
    @Autowired
    private IUserExamQuestionService userExamQuestionService;
    @Autowired
    private IQuestionService questionService;
    @Autowired
    private IUserExamResultService userExamResultService;
    @Autowired
    private IQuartzJobService quartzJobService;

    @Autowired
    private ISysDepartService sysDepartService;

    @Autowired
    private IUserExamQuestionAnswerService userExamQuestionAnswerService;

    @Autowired
    private IPaperRuleGroupService paperRuleGroupService;
    @Autowired
    private ExamPapersServiceImpl examPapersService;
    @Autowired
    private IPaperService iPaperService;
    @Autowired
    private IUserExamPapersService iUserExamPapersService;
    @Autowired
    private IExamPapersService iExamPapersService;

    @Override
    public Exam detail(String id) {
        return examMapper.detail(id);
    }

    /**
     * 分页查询
     */
    @Override
    public IPage<Exam> selectExamList(IPage<Exam> page, Exam exam, LoginUser user) {
        return examMapper.selectExamList(page, exam, user);
    }

    @Override
    public IPage<Exam> selectExamList2(IPage<Exam> page, Exam exam, LoginUser user) {
        return examMapper.selectExamList2(page, exam, user);
    }


    /**
     * 创建考试
     *
     * @param userId
     * @param examId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public UserExam createExam(String userId, String examId) {
        Exam exam = detail(examId);
        if (exam == null) {
            throw new JeecgBootException("考试不存在！");
        }
        if (!ExamState.ENABLE.equals(exam.getState())) {
            throw new JeecgBootException("不在考试时间！");
        }
        if (!Objects.isNull(exam.getLimitCount())) {
            long userExamNum = userExamService.countUserExam(userId, examId);
            if (userExamNum >= exam.getLimitCount()) {
                throw new JeecgBootException("考试次数达到限制！");
            }
        }
        Paper paper = exam.getPaper();
        if (Objects.isNull(paper)) {
            throw new JeecgBootException("试卷不存在！");
        }
        List<PaperQuestion> paperQuestionList = null;
        if (paper.getJoinType().equals(JoinType.XT.getCode()) || paper.getJoinType().equals(JoinType.CT.getCode())) {//选题或者抽题
            //只查询父题目
            paperQuestionList = paperQuestionService.listByPaperId(paper.getId());
            //题目乱序
            if (!Objects.isNull(exam.getQuestionDisorder()) && exam.getQuestionDisorder()) {
                //题目洗牌
                paperQuestionList = questionShuffle(paperQuestionList);
            }
        } else if (paper.getJoinType().equals(JoinType.SJ.getCode())) {//随机抽题
            //生成题目
            paperQuestionList = paperService.generateQuestion(paper.getId());
        }
        if (CollectionUtils.isEmpty(paperQuestionList)) {
            throw new JeecgBootException("无对应的考题！");
        }
        //保存考试
        return this.saveExam(userId, exam, paper, paperQuestionList, paper.getJoinType());
    }
    /**
     * 创建考试与试卷
     *
     * @param userId
     * @param examId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public UserExam createExamPapers(String userId, String examId) {
        Exam exam = detail(examId);
        if (exam == null) {
            throw new JeecgBootException("考试不存在！");
        }
        if (!ExamState.ENABLE.equals(exam.getState())) {
            throw new JeecgBootException("不在考试时间！");
        }
        if (!Objects.isNull(exam.getLimitCount())) {
            long userExamNum = userExamService.countUserExam(userId, examId);
            if (userExamNum >= exam.getLimitCount()) {
                throw new JeecgBootException("考试次数达到限制！");
            }
        }
        //校验是否有正在进行的考试
        UserExam userExam = userExamService.listExamIn(userId);
        List<UserExamPapers>  userExamPapersList=new ArrayList<>();
        //获取需要考试的试卷  需要判断有几套试卷,同时有几套试卷开始考了
        List<ExamPapers> examPapersList= examPapersService.ExamIdAllExamPapers(examId);
        //试卷
        Paper paper=new Paper();
        if (userExam==null) {
            //如何用户考试记录为空从第一套试卷开始考
            paper=iPaperService.getById(examPapersList.get(0).getPaperId());
        }else {
            //检查是否有正在进行中的考试
            //获取有没有正在考试的试卷
            LambdaQueryWrapper<UserExamPapers> wrapperpaper = new LambdaQueryWrapper<UserExamPapers>();
            wrapperpaper.eq(UserExamPapers::getUserExamId, userExam.getId())
                    .eq(UserExamPapers::getState, PaperState.ING);
            userExamPapersList=BeanContext.getApplicationContext().getBean(IUserExamPapersService.class).list(wrapperpaper);
        if (userExamPapersList.size()>0){
            throw new JeecgBootException("有正在进行的考试！");
        }
            //获取已经考了几张试卷开始考下一张
            paper=iPaperService.getById(examPapersList.get(iUserExamPapersService.UserExamIdAllUserExamPapers2(userExam.getId()).size()).getPaperId());
        }

//        //如果没有考试的试卷创建第一套试卷
//        if (userExamPapersList.size() == 0) {
//            System.out.println("yyz如果没有考试的试卷创建第一套试卷:"+userExamPapersList);
//            paper=iPaperService.getById(examPapersList.get(0).getPaperId());
//        }else {
//            //获取考了几套试卷
//            paper=iPaperService.getById(examPapersList.get(userExamPapersList.size()).getPaperId());
//        }
        //获取考试绑定的试题详情
        List<ExamPapers> PaperIdAllExamPapers=examPapersService.PaperIdAllExamPapers(examId,paper.getId());
        List<PaperQuestion> paperQuestionList = null;
        if (paper.getJoinType().equals(JoinType.XT.getCode()) || paper.getJoinType().equals(JoinType.CT.getCode())) {//选题或者抽题
            //只查询父题目
            paperQuestionList = paperQuestionService.listByPaperId(paper.getId());
            //题目乱序
            if (!Objects.isNull(exam.getQuestionDisorder()) && exam.getQuestionDisorder()) {
                //题目洗牌
                paperQuestionList = questionShuffle(paperQuestionList);
            }
        } else if (paper.getJoinType().equals(JoinType.SJ.getCode())) {//随机抽题
            //生成题目
            paperQuestionList = paperService.generateQuestion(paper.getId());
        }
        if (CollectionUtils.isEmpty(paperQuestionList)) {
            throw new JeecgBootException("无对应的考题！");
        }
        //保存考试
        return this.saveExamPapers(userId, exam, paper, paperQuestionList, paper.getJoinType(),PaperIdAllExamPapers,userExam);
    }

    /**
     * 题目乱序
     */
    public List<PaperQuestion> questionShuffle(List<PaperQuestion> paperQuestionList) {
        List<PaperQuestion> shuffleList = new ArrayList<>();
        Map<Integer, List<PaperQuestion>> map = paperQuestionList.stream().collect(Collectors.groupingBy(PaperQuestion::getQuestionType, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<Integer, List<PaperQuestion>> item : map.entrySet()) {
            //题目洗牌
            Collections.shuffle(item.getValue());
            shuffleList.addAll(item.getValue());
        }
        //重新排序
        return paperQuestionService.sort(shuffleList);
    }


    /**
     * 保存考试
     *
     * @param userId
     * @param exam
     * @param paperQuestionList
     * @return
     */
    public UserExam saveExam(String userId, Exam exam, Paper paper, List<PaperQuestion> paperQuestionList, Integer joinType) {
        UserExam userExam = new UserExam();
        userExam.setId(IdUtil.getSnowflake(1, 1).nextIdStr());
        userExam.setExamId(exam.getId());
        userExam.setUserScore(0.0);
        userExam.setUserId(userId);
        userExam.setPaperId(paper.getId());
        userExam.setState(PaperState.ING);
        userExam.setTotalScore(exam.getTotalScore());
        userExam.setQualifyScore(exam.getQualifyScore());
        userExam.setHasSubjective(paper.getHasSubjective());//是否有主观题

        //保存考试关联的试题列表
        if (!CollectionUtils.isEmpty(paperQuestionList)) {
            //保存用户考试题目，如果有组合题，则也要保存子题目
            this.savePaperQuestion(paper.getId(), userExam.getId(), paperQuestionList, joinType);
            //保存用户考试的题目答案
            this.saveQuestionAnswer(paper.getId(), userExam.getId(), exam, paperQuestionList);
        }

        // 截止时间
        Calendar cl = Calendar.getInstance();
        cl.setTimeInMillis(System.currentTimeMillis());
        cl.add(Calendar.MINUTE, exam.getTotalTime());
        userExam.setLimitTime(cl.getTime());

        //保存考试记录
        userExamService.save(userExam);

        //异步创建定时任务，到交卷时间的10分钟后强制交卷
        ThreadPoolUtil.getExecutor().execute(() -> this.createJob(userExam.getId(), userExam.getLimitTime()));
        return userExam;
    }
    /**
     * 保存考试与试卷
     *
     * @param userId
     * @param exam
     * @param paperQuestionList
     * @return
     */
    public UserExam saveExamPapers(String userId, Exam exam, Paper paper, List<PaperQuestion> paperQuestionList, Integer joinType,List<ExamPapers> PaperIdAllExamPapers,UserExam userExam) {
        ExamPapers examPapers=PaperIdAllExamPapers.get(0);
        //组建用户考试记录对象
//        UserExam userExam = new UserExam();
        if (userExam == null) {
            userExam = new UserExam();
            userExam.setId(IdUtil.getSnowflake(1, 1).nextIdStr());
        }
        userExam.setExamId(exam.getId());
        userExam.setUserScore(0.0);
        userExam.setUserId(userId);
        userExam.setPaperId(paper.getId());
        userExam.setState(PaperState.ING);
        userExam.setTotalScore(exam.getTotalScore());
        userExam.setQualifyScore(exam.getQualifyScore());
        userExam.setHasSubjective(paper.getHasSubjective());//是否有主观题

        //保存考试关联的试题列表
        if (!CollectionUtils.isEmpty(paperQuestionList)) {
            //保存用户考试题目，如果有组合题，则也要保存子题目
            this.savePaperQuestionExamId(paper.getId(), userExam.getId(), paperQuestionList, joinType);
            //保存用户考试的题目答案
            this.saveQuestionAnswersExamId(paper.getId(), userExam.getId(), exam, paperQuestionList);
        }

        // 截止时间
        Calendar cl = Calendar.getInstance();
        cl.setTimeInMillis(System.currentTimeMillis());
        cl.add(Calendar.MINUTE, exam.getTotalTime());
        userExam.setLimitTime(cl.getTime());
        //组件用户考试试卷记录对象
        UserExamPapers userExamPapers=new UserExamPapers();
        userExamPapers.setId(IdUtil.getSnowflake(1, 1).nextIdStr());
        userExamPapers.setUserScore(0.0);
        userExamPapers.setPaperId(paper.getId());
        userExamPapers.setState(PaperState.ING);
        userExamPapers.setTotalScore(exam.getTotalScore());
        userExamPapers.setQualifyScore(exam.getQualifyScore());
        userExamPapers.setHasSubjective(paper.getHasSubjective());//是否有主观题
        // 试卷截止时间
        Calendar cl1 = Calendar.getInstance();
        cl1.setTimeInMillis(System.currentTimeMillis());
        cl1.add(Calendar.MINUTE, examPapers.getPaperTime());
        userExamPapers.setLimitTime(cl1.getTime());
        //判断创建的是第几套考试 如果是第一套试卷保存第一套考试记录以及保存试卷记录
        if (examPapers.getPaperOrder().equals(1)) {
            //保存考试记录
            userExamService.save(userExam);
            //用户考试记录id
            userExamPapers.setUserExamId(userExam.getId());
            //保存试卷记录
            iUserExamPapersService.save(userExamPapers);
            //        异步创建定时任务，到交卷时间的10分钟后强制交卷
            UserExam finalUserExam = userExam;
            ThreadPoolUtil.getExecutor().execute(() -> this.createPapersJob(finalUserExam.getId()+finalUserExam.getPaperId(), userExamPapers.getLimitTime()));
        }else {//只保存试卷记录
            //获取正在考试中的用户考试记录
            userExamPapers.setUserExamId(userExamService.listExamIn(userId).getId());
            //保存试卷记录
            iUserExamPapersService.save(userExamPapers);
            //        异步创建定时任务，到交卷时间的10分钟后强制交卷
            UserExam finalUserExam = userExam;
            ThreadPoolUtil.getExecutor().execute(() -> this.createPapersJob(finalUserExam.getId()+finalUserExam.getPaperId(), userExamPapers.getLimitTime()));
        }
        List<UserExamPapers> userExamPapersList=new ArrayList<>();
        userExamPapersList.add(userExamPapers);
        userExam.setUserExamPapersList(userExamPapersList);
        return userExam;
    }
    /**
     * 保存用户考试的题目答案
     *
     * @param userExamId
     * @param paperQuestionList
     */
    public void saveQuestionAnswer(String paperId, String userExamId, Exam exam, List<PaperQuestion> paperQuestionList) {
        //组装试卷试题，子题目，答案
        List<Question> questionList = paperService.assemblePaperQuestion(paperId, paperQuestionList, null, null);
        if (!CollectionUtils.isEmpty(questionList)) {
            List<UserExamQuestionAnswer> userExamQuestionAnswerList = new ArrayList<>();
            questionList.forEach(question -> {
                if (QuestionType.COMBINATION.getCode().equals(question.getType())) {//组合题
                    List<Question> subQuestionList = question.getSubQuestionList();
                    if (!CollectionUtils.isEmpty(subQuestionList)) {
                        subQuestionList.forEach(subQuestion -> genQuestionAnswer(userExamId, exam, subQuestion, userExamQuestionAnswerList));
                    }
                } else {
                    genQuestionAnswer(userExamId, exam, question, userExamQuestionAnswerList);
                }
            });
            userExamQuestionAnswerService.saveBatch(userExamQuestionAnswerList);
        }
    }
    /**
     * yyz保存用户考试的题目答案
     *
     * @param userExamId
     * @param paperQuestionList
     */
    public void saveQuestionAnswersExamId(String paperId, String userExamId, Exam exam, List<PaperQuestion> paperQuestionList) {
        //组装试卷试题，子题目，答案
        List<Question> questionList = paperService.assemblePaperQuestion(paperId, paperQuestionList, null, null);
        if (!CollectionUtils.isEmpty(questionList)) {
            List<UserExamQuestionAnswer> userExamQuestionAnswerList = new ArrayList<>();
            questionList.forEach(question -> {
                if (QuestionType.COMBINATION.getCode().equals(question.getType())) {//组合题
                    List<Question> subQuestionList = question.getSubQuestionList();
                    if (!CollectionUtils.isEmpty(subQuestionList)) {
                        subQuestionList.forEach(subQuestion -> genQuestionAnswer(userExamId, exam, subQuestion, userExamQuestionAnswerList));
                    }
                } else {
                    genQuestionAnswer(userExamId, exam, question, userExamQuestionAnswerList);
                }
            });
            //添加试卷id
            for (int i = 0; i < userExamQuestionAnswerList.size(); i++) {
                userExamQuestionAnswerList.get(i).setPaperId(paperId);
            }
            userExamQuestionAnswerService.saveBatch(userExamQuestionAnswerList);
        }
    }

    public void genQuestionAnswer(String userExamId, Exam exam, Question question,
                                  List<UserExamQuestionAnswer> userExamQuestionAnswerList) {
        List<QuestionAnswer> answerList = question.getAnswerList();
        if (!CollectionUtils.isEmpty(answerList)) {
            //答案乱序(支持单选题，多选题)
            if (!Objects.isNull(exam.getAnswerDisorder()) && exam.getAnswerDisorder() && (QuestionType.RADIO.getCode().equals(question.getType()) || QuestionType.MULTI.getCode().equals(question.getType()))) {
                //答案洗牌
                Collections.shuffle(answerList);
            }
            int sort = 1;
            for (QuestionAnswer questionAnswer : answerList) {
                UserExamQuestionAnswer userExamQuestionAnswer = new UserExamQuestionAnswer();
                userExamQuestionAnswer.setUserExamId(userExamId);
                userExamQuestionAnswer.setQuestionId(question.getId());
                userExamQuestionAnswer.setAnswerId(questionAnswer.getId());
                userExamQuestionAnswer.setIsRight(questionAnswer.getIsRight());
                userExamQuestionAnswer.setSort(sort);
                //填空题、连线题、拖拽题不再生成tag
                if (QuestionType.BLANK.getCode().equals(question.getType())||
                    QuestionType.DRAG.getCode().equals(question.getType())||
                    QuestionType.CONNECTION.getCode().equals(question.getType())) {
                    userExamQuestionAnswer.setTag(questionAnswer.getTag());
                } else {
                    userExamQuestionAnswer.setTag(CharUtil.getZm(sort - 1));
                }
                userExamQuestionAnswerList.add(userExamQuestionAnswer);
                sort++;
            }
        }
    }


    /**
     * 创建定时任务
     *
     * @param userExamId
     * @param limitTime
     */
    public void createJob(String userExamId, Date limitTime) {
        // 强制交卷任务
        QuartzJob quartzJob = new QuartzJob();

        //交卷时间的10分钟后强制交卷
        DateTime offsetTime = DateUtil.offsetMinute(limitTime, 10);
        quartzJob.setCronExpression(CronUtils.dateToCron(offsetTime));
        quartzJob.setJobClassName(BreakExamJob.class.getName());
        quartzJob.setParameter(userExamId);
        quartzJob.setStatus(CommonConstant.STATUS_NORMAL);
        quartzJobService.saveAndScheduleJob(quartzJob);
    }
    /**
     * 创建定时任务强制考试与试卷交卷
     *
     * @param userExamId
     * @param limitTime
     */
    public void createPapersJob(String userExamId, Date limitTime) {
        // 强制交卷任务
        QuartzJob quartzJob = new QuartzJob();

        //交卷时间的10分钟后强制交卷
        DateTime offsetTime = DateUtil.offsetMinute(limitTime, 10);
        quartzJob.setCronExpression(CronUtils.dateToCron(offsetTime));
        quartzJob.setJobClassName(BreakExamJob.class.getName());
        quartzJob.setParameter(userExamId);
        quartzJob.setStatus(CommonConstant.STATUS_NORMAL);
        quartzJobService.saveAndScheduleJob(quartzJob);
    }
    /**
     * 获取规则组map
     *
     * @param paperQuestionList
     * @return
     */
    public Map<String, PaperRuleGroup> getGroupMap(List<PaperQuestion> paperQuestionList) {
        List<String> groupIdList = paperQuestionList.stream().map(PaperQuestion::getGroupId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(groupIdList)) {
            List<PaperRuleGroup> groupList = paperRuleGroupService.listByIds(groupIdList);
            return groupList.stream().collect(Collectors.toMap(PaperRuleGroup::getId, a -> a, (k1, k2) -> k1));
        }
        return null;
    }

    /**
     * 保存用户考试题目，如果有组合题，则也要保存子题目
     *
     * @param paperQuestionList 主题目
     */
    private void savePaperQuestion(String paperId, String userExamId, List<PaperQuestion> paperQuestionList, Integer joinType) {
        paperQuestionList = paperQuestionList.stream().sorted(Comparator.comparing(PaperQuestion::getQuestionType)
                .thenComparing(PaperQuestion::getSort)).collect(Collectors.toList());
        //获取规则组map
        Map<String, PaperRuleGroup> groupMap = getGroupMap(paperQuestionList);

        List<UserExamQuestion> userExamQuestionList = new ArrayList<>();
        int index = 1;
        for (PaperQuestion item : paperQuestionList) {
            //配置用户考试试题
            configUserExamQuestion(userExamId, item, index, groupMap, userExamQuestionList);
            index++;
        }

        //组合题
        List<String> combinationIdList = paperQuestionList.stream().filter(r -> QuestionType.COMBINATION.getCode().equals(r.getQuestionType()))
                .map(PaperQuestion::getQuestionId).collect(Collectors.toList());
        List<PaperQuestion> paperSubQuestionList = null;
        if (!CollectionUtils.isEmpty(combinationIdList)) {
            if (joinType.equals(JoinType.XT.getCode()) || joinType.equals(JoinType.CT.getCode())) {//选题或者抽题
                paperSubQuestionList = paperQuestionService.listPaperSubQuestionByIds(paperId, combinationIdList);
            } else if (joinType.equals(JoinType.SJ.getCode())) {//随机抽题
                paperSubQuestionList = new ArrayList<>();
                List<Question> subQuestionList = questionService.listSubQuestionNoAnswerByIds(combinationIdList);
                Map<String, Long> subQuestionCountMap = subQuestionList.stream().collect(Collectors.groupingBy(Question::getParentId, Collectors.counting()));
                for (Question sub : subQuestionList) {
                    PaperQuestion parent = paperQuestionList.stream().filter(r -> r.getQuestionId().equals(sub.getParentId())).findFirst().orElse(null);
                    PaperQuestion paperSubQuestion = getPaperQuestion(paperId, sub, parent, calcSubQuestionScore(subQuestionCountMap, parent));
                    paperSubQuestionList.add(paperSubQuestion);
                }
            }
        }

        //配置子题目
        configUserExamSubQuestion(userExamId, paperSubQuestionList, groupMap, userExamQuestionList);
        //保存用户考试题目，如果有组合题，则也要保存子题目
        userExamQuestionService.saveBatch(userExamQuestionList);
    }
    /**
     * 保存用户考试题目，如果有组合题，则也要保存子题目
     *
     * @param paperQuestionList 主题目
     */
    private void savePaperQuestionExamId(String paperId, String userExamId,List<PaperQuestion> paperQuestionList, Integer joinType) {
        paperQuestionList = paperQuestionList.stream().sorted(Comparator.comparing(PaperQuestion::getQuestionType)
                .thenComparing(PaperQuestion::getSort)).collect(Collectors.toList());
        //获取规则组map
        Map<String, PaperRuleGroup> groupMap = getGroupMap(paperQuestionList);

        List<UserExamQuestion> userExamQuestionList = new ArrayList<>();
        int index = 1;
        for (PaperQuestion item : paperQuestionList) {
            //配置用户考试试题
            configUserExamQuestion(userExamId, item, index, groupMap, userExamQuestionList);
            index++;
        }

        //组合题
        List<String> combinationIdList = paperQuestionList.stream().filter(r -> QuestionType.COMBINATION.getCode().equals(r.getQuestionType()))
                .map(PaperQuestion::getQuestionId).collect(Collectors.toList());
        List<PaperQuestion> paperSubQuestionList = null;
        if (!CollectionUtils.isEmpty(combinationIdList)) {
            if (joinType.equals(JoinType.XT.getCode()) || joinType.equals(JoinType.CT.getCode())) {//选题或者抽题
                paperSubQuestionList = paperQuestionService.listPaperSubQuestionByIds(paperId, combinationIdList);
            } else if (joinType.equals(JoinType.SJ.getCode())) {//随机抽题
                paperSubQuestionList = new ArrayList<>();
                List<Question> subQuestionList = questionService.listSubQuestionNoAnswerByIds(combinationIdList);
                Map<String, Long> subQuestionCountMap = subQuestionList.stream().collect(Collectors.groupingBy(Question::getParentId, Collectors.counting()));
                for (Question sub : subQuestionList) {
                    PaperQuestion parent = paperQuestionList.stream().filter(r -> r.getQuestionId().equals(sub.getParentId())).findFirst().orElse(null);
                    PaperQuestion paperSubQuestion = getPaperQuestion(paperId, sub, parent, calcSubQuestionScore(subQuestionCountMap, parent));
                    paperSubQuestionList.add(paperSubQuestion);
                }
            }
        }

        //配置子题目
        configUserExamSubQuestion(userExamId, paperSubQuestionList, groupMap, userExamQuestionList);
        //保存前添加试卷id
        for (UserExamQuestion userExamQuestion : userExamQuestionList) {
            userExamQuestion.setPaperId(paperId);
        }
        //保存用户考试题目，如果有组合题，则也要保存子题目
        userExamQuestionService.saveBatch(userExamQuestionList);
    }
    /**
     * 随机组卷组合题计算每个子题目分数
     *
     * @param subQuestionCountMap
     * @param parent
     * @return
     */
    public Double calcSubQuestionScore(Map<String, Long> subQuestionCountMap, PaperQuestion parent) {
        double subQuestionScore = 0;
        if (!Objects.isNull(parent) && !Objects.isNull(subQuestionCountMap)
                && !Objects.isNull(subQuestionCountMap.get(parent.getQuestionId()))) {
            //TODO 分数改为double
            subQuestionScore = NumberUtil.div(parent.getQuestionScore().doubleValue(),
                    subQuestionCountMap.get(parent.getQuestionId()).doubleValue(), 2);
        }
        return subQuestionScore;
    }

    private PaperQuestion getPaperQuestion(String paperId, Question sub, PaperQuestion parent, double subQuestionScore) {
        PaperQuestion paperSubQuestion = new PaperQuestion();
        paperSubQuestion.setGroupId(!Objects.isNull(parent) ? parent.getGroupId() : null);
        paperSubQuestion.setPaperId(paperId);
        paperSubQuestion.setQuestionId(sub.getId());
        paperSubQuestion.setQuestionType(sub.getType());
        paperSubQuestion.setChild(sub.getChild());
        paperSubQuestion.setQuestionScore(subQuestionScore);
        paperSubQuestion.setSort(sub.getSort());
        paperSubQuestion.setParentQuestionId(sub.getParentId());
        return paperSubQuestion;
    }

    /**
     * 配置用户考试试题
     *
     * @param userExamId
     * @param item
     * @param index
     * @param groupMap
     * @param userExamQuestionList
     */
    public void configUserExamQuestion(String userExamId, PaperQuestion item, int index,
                                       Map<String, PaperRuleGroup> groupMap, List<UserExamQuestion> userExamQuestionList) {
        UserExamQuestion userExamQuestion = new UserExamQuestion();
        BeanUtils.copyProperties(item, userExamQuestion);

        //根据规则组设置题目
        setQuestionByGroup(item, userExamQuestion, groupMap);

        userExamQuestion.setId(null);
        userExamQuestion.setQuestionIndex(index);
        userExamQuestion.setUserExamId(userExamId);
        userExamQuestionList.add(userExamQuestion);
    }

    /**
     * 根据规则组设置题目
     *
     * @param item
     * @param userExamQuestion
     * @param groupMap
     */
    public void setQuestionByGroup(PaperQuestion item, UserExamQuestion userExamQuestion, Map<String, PaperRuleGroup> groupMap) {
        if (!Objects.isNull(groupMap)) {
            PaperRuleGroup ruleGroup = groupMap.get(item.getGroupId());
            if (!Objects.isNull(ruleGroup)) {
                //设置题目是否允许漏选
                userExamQuestion.setCanMissOption(ruleGroup.getCanMissOption());
                //设置题目是否按空给分
                userExamQuestion.setCanBlankOption(ruleGroup.getCanBlankOption());
            }
        }
    }

    /**
     * 配置用户考试试题子题目
     *
     * @param userExamId
     * @param groupMap
     * @param userExamQuestionList
     */
    public void configUserExamSubQuestion(String userExamId, List<PaperQuestion> paperSubQuestionList,
                                          Map<String, PaperRuleGroup> groupMap, List<UserExamQuestion> userExamQuestionList) {
        if (!CollectionUtils.isEmpty(paperSubQuestionList)) {
            paperSubQuestionList.forEach(subQuestion -> {
                UserExamQuestion userExamQuestion = new UserExamQuestion();
                BeanUtils.copyProperties(subQuestion, userExamQuestion);
                //根据规则组设置题目
                setQuestionByGroup(subQuestion, userExamQuestion, groupMap);
                userExamQuestion.setId(null);
                userExamQuestion.setChild(true);
                userExamQuestion.setParentQuestionId(subQuestion.getParentQuestionId());
                userExamQuestion.setQuestionIndex(subQuestion.getSort());
                userExamQuestion.setUserExamId(userExamId);
                userExamQuestionList.add(userExamQuestion);
            });
        }
    }


    /**
     * 在线考试试卷详情，包括答题卡，试题（不包含正确答案和解析）
     *
     * @param userExamId
     * @return
     */
    @Override
    public UserExamDTO examDetail(String userExamId) {
        return examDetail(userExamId, false, false, true, false);
    }

    public UserExamDTO examDetail(String userExamId, boolean needAnswerFlag, boolean needAnalysis, boolean needClearBlankContent, boolean needRenderBlank) {
        UserExamDTO dto = new UserExamDTO();
        UserExam userExam = userExamService.listUserExam(userExamId);
        //显示试卷详情
        if (Objects.isNull(userExam)) {
            throw new JeecgBootException("考试不存在！");
        }
        BeanUtils.copyProperties(userExam, dto);
        List<UserExamQuestion> userExamQuestionList = dto.getUserExamQuestionList();
        List<Map<String, AnswerCardDTO>> answerCardList = new ArrayList<>();
        List< AnswerCardDTO> answerCardList2 = new ArrayList<>();
        if (!CollectionUtils.isEmpty(userExamQuestionList)) {
            List<UserExamQuestion> rootQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && !r.getChild())
                    .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
            List<UserExamQuestion> childQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && r.getChild())
                    .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(rootQuestionList)) {
                //获取题目和答案
                assembleUserExamQuestion(userExamId, rootQuestionList, childQuestionList, needAnswerFlag, needAnalysis, false, true, needClearBlankContent);

                //设置题目
                setUserExamQuestion(rootQuestionList, needRenderBlank);

                //设置答题卡
//                setAnswerCard(rootQuestionList, answerCardList);
                setAnswerCard2(rootQuestionList, answerCardList2);
                dto.setUserExamQuestionList(rootQuestionList);
            }
        }
//        dto.setAnswerCardList(answerCardList);
        dto.setAnswerCardList2(answerCardList2);
        //获取有没有正在考试的试卷信息
        LambdaQueryWrapper<UserExamPapers> wrapperpaper = new LambdaQueryWrapper<UserExamPapers>();
        wrapperpaper.eq(UserExamPapers::getUserExamId, userExam.getId())
                .eq(UserExamPapers::getState, PaperState.ING);
        List<UserExamPapers> userExamPapersList=BeanContext.getApplicationContext().getBean(IUserExamPapersService.class).list(wrapperpaper);
        //获取本地当前时间
        userExamPapersList.get(0).setSystemTime(new Date());
        //在返回值中添加考试试卷信息
        dto.setUserExamPapersList(userExamPapersList);
        //添加当前考试的所有试卷信息
        List<ExamPapers> examPapersList=iExamPapersService.getExamIdPaper(userExam.getExamId());
        List<UserExamPapers> userExamPapersList1=new ArrayList<UserExamPapers>();
        for (ExamPapers examPapers : examPapersList) {
            //获取是否考试状态并赋值
            userExamPapersList1 = iUserExamPapersService.UserExamPapersList(userExamId, examPapers.getPaperId());
            if (userExamPapersList1.size() == 0) {
                examPapers.setState(9);
            } else {
                examPapers.setState(userExamPapersList1.get(0).getState());
            }

        }

        dto.setExamPapersList(examPapersList);
        return dto;
    }

    /**
     * 用户考试成绩详情，包括答题卡，试题（包含正确答案和解析）
     *
     * @param userExamId
     * @return
     */
    @Override
    public UserExamDTO userExamResultDetail(String userExamId) {
        UserExamDTO userExamDTO = examDetail(userExamId, true, true, false, true);
        //设置单选题/多选题/判断题的答案tag
        setAnswerTag(userExamDTO);
        return userExamDTO;
    }

    /**
     * 设置单选题/多选题/判断题的答案tag
     *
     * @param userExamDTO
     */
    public void setAnswerTag(UserExamDTO userExamDTO) {
        List<UserExamQuestion> userExamQuestionList = userExamDTO.getUserExamQuestionList();
        if (!CollectionUtils.isEmpty(userExamQuestionList)) {
            userExamQuestionList.forEach(userExamQuestion -> {
                if (QuestionType.COMBINATION.getCode().equals(userExamQuestion.getQuestionType())) {//组合题
                    userExamQuestion.getSubQuestionList().forEach(o -> setTagByAnswerIds(o));
                } else {
                    setTagByAnswerIds(userExamQuestion);
                }
            });
        }
    }

    /**
     * 根据单选题/多选题/判断题的答案ID回写tag
     *
     * @param userExamQuestion
     */
    public void setTagByAnswerIds(UserExamQuestion userExamQuestion) {
        if (QuestionType.RADIO.getCode().equals(userExamQuestion.getQuestionType())
                || QuestionType.MULTI.getCode().equals(userExamQuestion.getQuestionType())
                || QuestionType.JUDGE.getCode().equals(userExamQuestion.getQuestionType())) {
            userExamQuestion.setAnswer(generateQuestionAnswerTag(userExamQuestion.getAnswer(), userExamQuestion.getQuestion()));
        }
    }

    /**
     * 生成单选，单选，填空答案tag
     *
     * @param userAnswer
     * @param question
     * @return
     */
    @Override
    public String generateQuestionAnswerTag(String userAnswer, Question question) {
        if (Objects.isNull(question)) {
            return null;
        }

        List<QuestionAnswer> answerList = question.getAnswerList();
        if (StringUtils.isNotBlank(userAnswer) && !CollectionUtils.isEmpty(answerList)) {
            List<String> answers = Arrays.asList(userAnswer.split(","));
            List<QuestionAnswer> resultAnswerList = answerList.stream().filter(item -> answers.contains(item.getId())).collect(Collectors.toList());
            List<String> tagList = resultAnswerList.stream().map(QuestionAnswer::getTag).collect(Collectors.toList());
            return tagList.stream().sorted().collect(Collectors.joining(","));
        }
        return null;
    }

    /**
     * 设置题目
     *
     * @param userExamQuestionList
     */
    public void setUserExamQuestion(List<UserExamQuestion> userExamQuestionList, boolean needRenderBlank) {
        //与问题关联
        for (UserExamQuestion userExamQuestion : userExamQuestionList) {
            Question question = userExamQuestion.getQuestion();
            //渲染填空题的题干，填充进答案
            if (QuestionType.COMBINATION.getCode().equals(question.getType())) {//组合题
                userExamQuestion.getSubQuestionList().forEach(subQuestion -> {
                    rendBlankAnswer(needRenderBlank, subQuestion, subQuestion.getQuestion());
                    subQuestion.setQuestionTypeName(QuestionType.getByCode(subQuestion.getQuestionType()).getValue());
                });
            } else {
                rendBlankAnswer(needRenderBlank, userExamQuestion, question);
            }
            userExamQuestion.setQuestion(question);
            userExamQuestion.setQuestionTypeName(QuestionType.getByCode(userExamQuestion.getQuestionType()).getValue());
        }
    }

    /**
     * 渲染填空题、拖拽题、连线题答案
     *
     * @param needRenderBlank
     * @param userExamQuestion
     * @param question
     */
    public void rendBlankAnswer(boolean needRenderBlank, UserExamQuestion userExamQuestion, Question question) {
        if ((needRenderBlank && QuestionType.BLANK.getCode().equals(userExamQuestion.getQuestionType()))||
            (needRenderBlank && QuestionType.DRAG.getCode().equals(userExamQuestion.getQuestionType()))||
            (needRenderBlank && QuestionType.CONNECTION.getCode().equals(userExamQuestion.getQuestionType()))) {
            renderBlank(question, userExamQuestion.getAnswer());
        }
    }

    /**
     * 渲染填空题的题干，填充进答案
     *
     * @param question
     */
    @Override
    public void renderBlank(Question question, String answer) {
        String content = question.getContent();
        if (StringUtils.isNotBlank(content) && StringUtils.isNotBlank(answer)) {
            List<QuestionAnswer> answerList = JSONArray.parseArray(answer, QuestionAnswer.class);
            Map<String, QuestionAnswer> answerMap = answerList.stream().collect(Collectors.toMap(QuestionAnswer::getTag, a -> a, (k1, k2) -> k1));
            for (QuestionAnswer a : answerList) {
                String fontColor = !Objects.isNull(a.getIsRight()) && a.getIsRight() ? "#00bb00" : "red";
                content = content.replaceAll("\\>" + a.getTag() + "\\<\\/span\\>", " style=\"color:" + fontColor + ";font-weight:bold\">" + answerMap.get(a.getTag()).getContent() + "</span>");
            }
        }
        question.setContent(content);
    }

    /**
     * 设置答题卡
     *
     * @param userExamQuestionList
     * @param answerCardList
     */
    public void setAnswerCard(List<UserExamQuestion> userExamQuestionList, List<Map<String, AnswerCardDTO>> answerCardList) {
        Map<Integer, List<UserExamQuestion>> map = userExamQuestionList.stream().collect(Collectors.groupingBy(UserExamQuestion::getQuestionType, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<Integer, List<UserExamQuestion>> item : map.entrySet()) {
            AnswerCardDTO answerCard = new AnswerCardDTO();
            List<UserExamQuestion> questions = item.getValue();
            answerCard.setQuestionType(item.getKey());
            //题数和分数
            answerCard.setQuestionCount(questions.size());
            answerCard.setQuestionScore(questions.stream().mapToDouble(UserExamQuestion::getQuestionScore).sum());
            //答题卡数字面板
            answerCard.setIndexList(questions.stream().map(UserExamQuestion::getQuestionIndex).collect(Collectors.toList()));
            Map<String, AnswerCardDTO> answerCardMap = new LinkedHashMap<>();
            answerCardMap.put(QuestionType.getByCode(item.getKey()).getValue(), answerCard);
            answerCardList.add(answerCardMap);
        }
    }
    /**
     * 设置答题卡2
     *
     * @param userExamQuestionList
     * @param answerCardList
     */
    public void setAnswerCard2(List<UserExamQuestion> userExamQuestionList, List<AnswerCardDTO> answerCardList) {
        Map<Integer, List<UserExamQuestion>> map = userExamQuestionList.stream().collect(Collectors.groupingBy(UserExamQuestion::getQuestionType, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<Integer, List<UserExamQuestion>> item : map.entrySet()) {
            AnswerCardDTO answerCard = new AnswerCardDTO();
            List<UserExamQuestion> questions = item.getValue();
            answerCard.setQuestionType(item.getKey());
            //题数和分数
            answerCard.setQuestionCount(questions.size());
            answerCard.setQuestionScore(questions.stream().mapToDouble(UserExamQuestion::getQuestionScore).sum());
            //答题卡数字面板
            answerCard.setIndexList(questions.stream().map(UserExamQuestion::getQuestionIndex).collect(Collectors.toList()));
            answerCard.setQuestionTypeName(QuestionType.getByCode(item.getKey()).getValue());
//            answerCardMap.put(QuestionType.getByCode(item.getKey()).getValue(), answerCard);
            answerCardList.add(answerCard);
        }
    }
    /**
     * yyz交卷
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String submitExamPaper(ExamSubmitDTO dto) {
        String biaozhi="";
        System.out.println("yyz进行交卷");
        if (StringUtils.isEmpty(dto.getUserExamId())) {
            throw new JeecgBootException("考试ID不能为空");
        }
        if (StringUtils.isEmpty(dto.getPaperId())) {
            throw new JeecgBootException("试卷ID不能为空");
        }
        //获取试卷是否已经进行交卷
        List<UserExamPapers> listuserExamPapers=iUserExamPapersService.UserExamIdPaperUserExamPapers(dto.getUserExamId(),dto.getPaperId());
        UserExamPapers userExamPapers=new UserExamPapers();
        if (listuserExamPapers.size()>0){
            userExamPapers=listuserExamPapers.get(0);
        }else {
            throw new JeecgBootException("未查询到该试卷");
        }
        if (PaperState.FINISHED.equals(userExamPapers.getState()) || PaperState.WAIT_OPT.equals(userExamPapers.getState())) {
            //试卷已经交卷获取下一场考试试卷的信息
           List<UserExamPapers> userExamPapersLists= iUserExamPapersService.UserExamIdAllUserExamPapers(dto.getUserExamId());
           if(userExamPapersLists.size()==0){
               throw new JeecgBootException("该试卷已经交卷，不允许重复交卷,当前试卷为最后一套试卷,本次考试结束。");
           }else if(userExamPapersLists.size()==1){
//               throw new JeecgBootException("该试卷已经交卷，不允许重复交卷,即将进入下一套试卷。",Integer.parseInt(userExamPapersLists.get(0).getPaperId()));
               System.out.println("yyz显示："+userExamPapersLists.get(0).getPaperId());
               return userExamPapersLists.get(0).getPaperId();
           }
        }
        UserExam userExam = userExamService.listUserExam(dto.getUserExamId());
        if (Objects.isNull(userExam)) {
            throw new JeecgBootException("考试记录不存在");
        }

        if (PaperState.FINISHED.equals(userExam.getState()) || PaperState.WAIT_OPT.equals(userExam.getState())) {
            throw new JeecgBootException("该考试已经交卷，不允许重复交卷");
        }
        List<UserExamQuestion> userExamQuestionList = userExam.getUserExamQuestionList();
        if (CollectionUtils.isEmpty(userExamQuestionList)) {
            throw new JeecgBootException("考题查询失败");
        }

        List<UserExamQuestion> rootQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && !r.getChild())
                .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
        List<UserExamQuestion> childQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && r.getChild())
                .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(rootQuestionList)) {
            //组装题目、子题目和答案
            assembleUserExamQuestion(dto.getUserExamId(), rootQuestionList, childQuestionList, true, false, true, false, false);
        }
        //如果没有作答
        List<ExamAnswerDTO> examAnswers = dto.getExamAnswers();
        if (CollectionUtils.isEmpty(examAnswers)) {
            //没作答0分
            biaozhi=updateUserExamPaperAndResult(0, userExam);
            //没作答时需要更新客观题
            updateObjectiveQuestion(userExamQuestionList);
        } else {
            //有作答
            Map<Integer, UserExamQuestion> userExamQuestionMap = rootQuestionList.stream().collect(Collectors.toMap(UserExamQuestion::getQuestionIndex, a -> a, (k1, k2) -> k1));

            //计算客观题得分
            double score = calcObjectiveScore(examAnswers, userExamQuestionMap, rootQuestionList);
            //更新用户考试信息和成绩
            biaozhi=updateUserExamPaperAndResult(score, userExam);
        }

        //异步删除自动交卷任务，清除redis缓存
        ThreadPoolUtil.getExecutor().execute(() -> {
            this.deleteAutoCommitJob(dto.getUserExamId()+dto.getPaperId());
            deleteCacheAnswer(dto.getUserExamId()+dto.getPaperId());
        });
        return biaozhi;
    }
    /**
     * yyz单个试卷交卷
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void submitExam(ExamSubmitDTO dto) {
        if (StringUtils.isEmpty(dto.getUserExamId())) {
            throw new JeecgBootException("考试ID不能为空");
        }
        UserExam userExam = userExamService.listUserExam(dto.getUserExamId());
        if (Objects.isNull(userExam)) {
            throw new JeecgBootException("考试记录不存在");
        }
        if (PaperState.FINISHED.equals(userExam.getState()) || PaperState.WAIT_OPT.equals(userExam.getState())) {
            throw new JeecgBootException("该考试已经交卷，不允许重复交卷");
        }
        List<UserExamQuestion> userExamQuestionList = userExam.getUserExamQuestionList();
        if (CollectionUtils.isEmpty(userExamQuestionList)) {
            throw new JeecgBootException("考题查询失败");
        }

        List<UserExamQuestion> rootQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && !r.getChild())
                .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
        List<UserExamQuestion> childQuestionList = userExamQuestionList.stream().filter(r -> !Objects.isNull(r.getChild()) && r.getChild())
                .sorted(Comparator.comparing(UserExamQuestion::getQuestionIndex)).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(rootQuestionList)) {
            //组装题目、子题目和答案
            assembleUserExamQuestion(dto.getUserExamId(), rootQuestionList, childQuestionList, true, false, true, false, false);
        }
        //如果没有作答
        List<ExamAnswerDTO> examAnswers = dto.getExamAnswers();
        if (CollectionUtils.isEmpty(examAnswers)) {
            updateUserExamAndResult(0, userExam);
            //没作答时需要更新客观题
            updateObjectiveQuestion(userExamQuestionList);
        } else {
            //有作答
            Map<Integer, UserExamQuestion> userExamQuestionMap = rootQuestionList.stream().collect(Collectors.toMap(UserExamQuestion::getQuestionIndex, a -> a, (k1, k2) -> k1));

            //计算客观题得分
            double score = calcObjectiveScore(examAnswers, userExamQuestionMap, rootQuestionList);
            //更新用户考试信息和成绩
            updateUserExamAndResult(score, userExam);
        }

        //异步删除自动交卷任务，清除redis缓存
        ThreadPoolUtil.getExecutor().execute(() -> {
            this.deleteAutoCommitJob(dto.getUserExamId());
            deleteCacheAnswer(dto.getUserExamId()+dto.getPaperId());
        });
    }

    /**
     * 没作答时需要更新客观题
     *
     * @param userExamQuestionList
     */
    public void updateObjectiveQuestion(List<UserExamQuestion> userExamQuestionList) {
        List<UserExamQuestion> objectiveList = userExamQuestionList.stream().filter(o -> !o.getQuestionType().equals(QuestionType.SIMPLE.getCode())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(objectiveList)) {
            objectiveList.stream().forEach(o -> {
                o.setAnswered(false);
                o.setIsRight(false);
            });
            userExamQuestionService.updateBatchById(objectiveList);
        }
    }

    //组装题目、子题目和答案
    @Override
    public void assembleUserExamQuestion(String userExamId, List<UserExamQuestion> rootExamQuestionList,
                                         List<UserExamQuestion> childQuestionList, boolean needAnswerFlag,
                                         boolean needAnalysis, boolean needPathScore, boolean needSubjective,
                                         boolean needClearBlankContent) {
        List<String> subjectiveList = new ArrayList<>();
        if (needSubjective) {//是否需要主观题
            List<String> rootSubjectList = getSubjectList(rootExamQuestionList);
            List<String> childSubjectList = getSubjectList(childQuestionList);
            if (!CollectionUtils.isEmpty(rootSubjectList)) {
                subjectiveList.addAll(rootSubjectList);
            }
            if (!CollectionUtils.isEmpty(childSubjectList)) {
                subjectiveList.addAll(childSubjectList);
            }
        }

        //客观题
        List<String> objectiveList = new ArrayList<>();
        List<String> rootObjectiveList = getObjectiveList(rootExamQuestionList);
        List<String> childObjectiveList = getObjectiveList(childQuestionList);
        if (!CollectionUtils.isEmpty(rootObjectiveList)) {
            objectiveList.addAll(rootObjectiveList);
        }
        if (!CollectionUtils.isEmpty(childObjectiveList)) {
            objectiveList.addAll(childObjectiveList);
        }
        List<Question> resultList = new ArrayList<>();
        List<Question> questionList = questionService.listUserExamQuestionAnswer(userExamId, objectiveList,
                subjectiveList, needAnswerFlag, needAnalysis, needPathScore, needSubjective);

        List<String> combinationIdList = rootExamQuestionList.stream().filter(o -> o.getQuestionType()
                .equals(QuestionType.COMBINATION.getCode())).map(UserExamQuestion::getQuestionId).collect(toList());
        List<Question> combinationQuestionList = null;
        if (!CollectionUtils.isEmpty(combinationIdList)) {
            combinationQuestionList = questionService.listByIds(combinationIdList);
        }
        if (!CollectionUtils.isEmpty(questionList)) {
            resultList.addAll(questionList);
        }
        if (!CollectionUtils.isEmpty(combinationQuestionList)) {
            resultList.addAll(combinationQuestionList);
        }
        //清空填空题答案并重新排序
        if (!CollectionUtils.isEmpty(resultList)) {
            resultList.forEach(question -> clearBlankAnswer(question, needClearBlankContent));
        }
        rootExamQuestionList.forEach(o -> {
            Question question = resultList.stream().filter(r -> r.getId().equals(o.getQuestionId())).findFirst().orElse(null);
            if (QuestionType.COMBINATION.getCode().equals(o.getQuestionType())) {//组合题
                List<UserExamQuestion> subQuestionList = childQuestionList.stream().filter(c -> StringUtils.isNotBlank(c.getParentQuestionId())
                                && c.getParentQuestionId().equals(o.getQuestionId())).sorted(Comparator.comparing(UserExamQuestion::getSort))
                        .collect(Collectors.toList());
                if (!Objects.isNull(question) && !CollectionUtils.isEmpty(subQuestionList)) {
                    subQuestionList.forEach(subQuestion -> {
                        Question tmp = resultList.stream().filter(r -> r.getId().equals(subQuestion.getQuestionId())).findFirst().orElse(null);
                        if (!Objects.isNull(tmp)) {
                            subQuestion.setQuestion(tmp);
                        }
                    });
                    o.setSubQuestionList(subQuestionList);
                }
                o.setQuestion(question);
            } else {
                if (!Objects.isNull(question)) {
                    o.setQuestion(question);
                }
            }
        });
    }

    /**
     * 获取主观题
     *
     * @param userExamQuestionList
     * @return
     */
    public List<String> getSubjectList(List<UserExamQuestion> userExamQuestionList) {
        return userExamQuestionList.stream().filter(o -> o.getQuestionType().equals(QuestionType.SIMPLE.getCode()))
                .map(UserExamQuestion::getQuestionId).collect(Collectors.toList());
    }

    public List<String> getObjectiveList(List<UserExamQuestion> userExamQuestionList) {
        return userExamQuestionList.stream().filter(o -> !o.getQuestionType().equals(QuestionType.SIMPLE.getCode())
                        && !o.getQuestionType().equals(QuestionType.COMBINATION.getCode())).map(UserExamQuestion::getQuestionId)
                .collect(Collectors.toList());
    }

    /**
     * 清空填空题答案
     *
     * @param question
     * @param needClearBlankContent
     */
    public void clearBlankAnswer(Question question, boolean needClearBlankContent) {
        List<QuestionAnswer> answerList = question.getAnswerList();
        if (!CollectionUtils.isEmpty(answerList)) {
            if ((QuestionType.BLANK.getCode().equals(question.getType()) && needClearBlankContent)||
                (QuestionType.DRAG.getCode().equals(question.getType()) && needClearBlankContent)||
                (QuestionType.CONNECTION.getCode().equals(question.getType()) && needClearBlankContent)) {//填空题、连线题、拖拽题是否清空答案内容
                answerList.forEach(o -> o.setContent(null));
            }
            question.setAnswerList(answerList.stream().sorted(Comparator.comparing(QuestionAnswer::getSort)).collect(Collectors.toList()));
        }
    }

    //获取题目和答案
    public List<Question> listQuestion(List<PaperQuestion> paperQuestionList) {
        List<String> questionIdList = paperQuestionList.stream().map(PaperQuestion::getQuestionId).collect(Collectors.toList());
        return questionService.selectQuestionList(questionIdList, true, true);
    }


    /**
     * 计算客观题得分
     *
     * @param examAnswers
     * @param userExamQuestionMap
     * @return
     */
    public double calcObjectiveScore(List<ExamAnswerDTO> examAnswers, Map<Integer, UserExamQuestion> userExamQuestionMap, List<UserExamQuestion> rootQuestionList) {
        if (!CollectionUtils.isEmpty(examAnswers)) {
            //对比/计算
            for (ExamAnswerDTO o : examAnswers) {
                //根据题型计算客观题得分
//                calcQuestionScoreByType(o, userExamQuestionMap);
                yyzjiacalcQuestionScoreByType(o, userExamQuestionMap);
            }
            List<UserExamQuestion> resultList = new ArrayList<>();
            rootQuestionList.forEach(root -> {
                if (QuestionType.COMBINATION.getCode().equals(root.getQuestionType())) {
                    long answerNum = examAnswers.stream().filter(answer -> answer.getIndex().startsWith(root.getQuestionIndex() + ".")).count();
                    if (answerNum > 0) {
                        root.setAnswered(true);
                    }
                    //组合题得分：从所有子题目得分求和
                    root.setActualScore(root.getSubQuestionList().stream().filter(o ->
                            !Objects.isNull(o.getActualScore())).mapToDouble(UserExamQuestion::getActualScore).sum());
                    resultList.add(root);
                    root.getSubQuestionList().forEach(subQuestion -> {
                        //是否是没回答的客观题,如果是，是否正确设置为false
                        noAnswerFill(examAnswers, root.getQuestionIndex() + "." + subQuestion.getQuestionIndex(), subQuestion);
                        resultList.add(subQuestion);
                    });
                    //子题目中是否包含简答题，如果包含简答题，需要阅卷
                    long simpleCount = root.getSubQuestionList().stream().filter(o -> o.getQuestionType()
                            .equals(QuestionType.SIMPLE.getCode())).count();
                    if (simpleCount > 0) {
                        root.setIsRight(null);
                    } else {
                        if (root.getActualScore() > 0) {
                            root.setIsRight(true);
                        } else {
                            root.setIsRight(false);
                        }
                    }
                } else {
                    noAnswerFill(examAnswers, String.valueOf(root.getQuestionIndex()), root);
                    resultList.add(root);
                }
            });
            if (!CollectionUtils.isEmpty(resultList)) {
                userExamQuestionService.updateBatchById(resultList);
            }
        }
        //最终得分
        return rootQuestionList.stream().filter(o -> !Objects.isNull(o.getActualScore()))
                .mapToDouble(UserExamQuestion::getActualScore).sum();
    }

    /**
     * 是否是没回答的客观题,如果是，是否正确设置为false
     *
     * @param examAnswers
     * @param questionIndex
     * @param userExamQuestion
     */
    public void noAnswerFill(List<ExamAnswerDTO> examAnswers, String questionIndex, UserExamQuestion userExamQuestion) {
        List<String> indexList = examAnswers.stream().map(ExamAnswerDTO::getIndex).collect(toList());
        if (!indexList.contains(questionIndex) && !userExamQuestion.getQuestionType().equals(QuestionType.SIMPLE.getCode())) {
            userExamQuestion.setIsRight(false);
        }
    }

    /**
     * 根据题型计算客观题得分
     *
     * @param o
     * @param userExamQuestionMap
     */
    public void calcQuestionScoreByType(ExamAnswerDTO o, Map<Integer, UserExamQuestion> userExamQuestionMap) {
        //根据题目序号中获取考题
        UserExamQuestion userExamQuestion = getUserExamQuestionByIndex(o.getIndex(), userExamQuestionMap);
        if (Objects.isNull(userExamQuestion)) {
            return;
        }
        userExamQuestion.setAnswered(true);//已答
        //主观题（简答题）
        if (userExamQuestion.getQuestionType().equals(QuestionType.SIMPLE.getCode())) {
            userExamQuestion.setAnswer(o.getSubjectiveAnswer());//答案内容
            return;
        }

        //单选，多选，判断题获取并设置答案
        String[] answers = o.getAnswers();
        if (!userExamQuestion.getQuestionType().equals(QuestionType.BLANK.getCode())) {
            if (!Objects.isNull(answers) && answers.length > 0) {
                userExamQuestion.setAnswer(String.join(",", answers));
            }
        }

        //填空题获取答案
        String blankAnswer = o.getBlankAnswer();

        //提交的答案和题目答案对比
        Question question = userExamQuestion.getQuestion();
        switch (QuestionType.getByCode(question.getType())) {
            //单选题和判断题直接答案完全匹配
            case RADIO:
            case JUDGE:
                compareToScore(userExamQuestion, answers, question);
                break;
            case MULTI: //多选题
                if (!Objects.isNull(userExamQuestion.getCanMissOption()) && userExamQuestion.getCanMissOption()) {
                    compareMultiToScore(userExamQuestion, answers, question);//漏选给分
                } else {
                    compareToScore(userExamQuestion, answers, question);
                }
                break;
            case BLANK: //填空题
                compareBlankToScore(userExamQuestion, blankAnswer, question, userExamQuestion.getCanBlankOption());
                break;
        }
    }
    /**
     * 根据题型计算客观题得分(包括连线题和拖拽题)
     *
     * @param o
     //* @param yyzjiacalcQuestionScoreByType
     */
    public void yyzjiacalcQuestionScoreByType(ExamAnswerDTO o, Map<Integer, UserExamQuestion> userExamQuestionMap) {
        //根据题目序号中获取考题
        UserExamQuestion userExamQuestion = getUserExamQuestionByIndex(o.getIndex(), userExamQuestionMap);
        if (Objects.isNull(userExamQuestion)) {
            return;
        }
        userExamQuestion.setAnswered(true);//已答
        //主观题（简答题）
        if (userExamQuestion.getQuestionType().equals(QuestionType.SIMPLE.getCode())) {
            userExamQuestion.setAnswer(o.getSubjectiveAnswer());//答案内容
            return;
        }

        //单选，多选，判断题获取并设置答案
//        RADIO(1, "单选题"),
//                MULTI(2, "多选题"),
//                JUDGE(3, "判断题"),
        String[] answers = o.getAnswers();
        if (userExamQuestion.getQuestionType().equals(QuestionType.RADIO.getCode())||
            userExamQuestion.getQuestionType().equals(QuestionType.MULTI.getCode())||
            userExamQuestion.getQuestionType().equals(QuestionType.JUDGE.getCode())) {
            if (!Objects.isNull(answers) && answers.length > 0) {
                userExamQuestion.setAnswer(String.join(",", answers));
            }
        }
//        if (!userExamQuestion.getQuestionType().equals(QuestionType.BLANK.getCode())) {
//            if (!Objects.isNull(answers) && answers.length > 0) {
//                userExamQuestion.setAnswer(String.join(",", answers));
//            }
//        }

        //填空题获取答案
        String blankAnswer = o.getBlankAnswer();

        //提交的答案和题目答案对比
        Question question = userExamQuestion.getQuestion();
        //switch (QuestionType.getByCode(question.getType()))
        switch (Objects.requireNonNull(QuestionType.getByCode(question.getType()))) {
            //单选题和判断题直接答案完全匹配
            case RADIO:
            case JUDGE:
                compareToScore(userExamQuestion, answers, question);
                break;
            case MULTI: //多选题
                if (!Objects.isNull(userExamQuestion.getCanMissOption()) && userExamQuestion.getCanMissOption()) {
                    compareMultiToScore(userExamQuestion, answers, question);//漏选给分
                } else {
                    compareToScore(userExamQuestion, answers, question);
                }
                break;
            case BLANK: //填空题
            case DRAG: //拖拽题
            case CONNECTION: //连线题
                compareBlankToScore(userExamQuestion, blankAnswer, question, userExamQuestion.getCanBlankOption());
                break;
        }
    }
    /**
     * 根据题目序号中获取考题
     *
     * @param index               题目序号
     * @param userExamQuestionMap 考题
     * @return
     */
    @Override
    public UserExamQuestion getUserExamQuestionByIndex(String index, Map<Integer, UserExamQuestion> userExamQuestionMap) {
        UserExamQuestion userExamQuestion = null;
//        System.out.println("yyz显示index："+index);
        if (index.indexOf(".") != -1) {//题目序号中有点号，则为组合题
            String[] indexArray = index.split("\\.");
            if (!Objects.isNull(indexArray) && indexArray.length > 1) {
                //获取组合题
                UserExamQuestion userExamRootQuestion = userExamQuestionMap.get(Integer.parseInt(indexArray[0]));
                if (!Objects.isNull(userExamRootQuestion)) {
                    //获取子题目
//                    System.out.println("获取子题目");
                    userExamQuestion = userExamRootQuestion.getSubQuestionList().stream().filter(item -> item.getQuestionIndex()
                            .equals(Integer.parseInt(indexArray[1]))).findFirst().orElse(null);
                }
            }
        } else {
            userExamQuestion = userExamQuestionMap.get(Integer.parseInt(index));
        }
        return userExamQuestion;
    }

    /**
     * 提交的答案和题目答案对比计算得分--填空题、拖拽题、连线题
     *
     * @return
     */
    public Double compareBlankToScore(UserExamQuestion userExamQuestion, String userAnswerStr, Question question, Boolean canBlankOption) {
        List<QuestionAnswer> answerList = question.getAnswerList();
        //深拷贝答案
        List<QuestionAnswer> deepList = deepCopyAnswerList(answerList);

        if (StringUtils.isNotBlank(userAnswerStr) && !CollectionUtils.isEmpty(answerList)) {
            List<QuestionAnswer> userAnswerList = JSONArray.parseArray(userAnswerStr, QuestionAnswer.class);
            //过滤无效答案
            userAnswerList = userAnswerList.stream().filter(ua -> StringUtils.isNotBlank(ua.getContent())).collect(Collectors.toList());
            List<BlankAnswerResultDTO> dtoList = new ArrayList<>();
            userAnswerList.stream().forEach(userAnswer -> {
                //严格按照顺序对比
                if (!Objects.isNull(question.getAnswerOrder()) && question.getAnswerOrder()) {
                    compareByOrder(answerList, userAnswer, dtoList);
                } else {
                    compareByNoOrder(answerList, userAnswer, dtoList);
                }
            });
            if (!CollectionUtils.isEmpty(dtoList)) {
                userExamQuestion.setAnswer(JSONArray.toJSONString(dtoList));//设置填空题答案
                if (dtoList.size() != deepList.size()) { //用户只作答了题目部分空，没答的空自然就是错的了（有错误答案）
                    return calcBlankScoreWhenFalse(userExamQuestion, canBlankOption, dtoList);
                } else {
                    //题目所有的填空用户都作答了
                    //找出答错的空
                    long falseNum = dtoList.stream().filter(d -> !Objects.isNull(d.getIsRight()) && !d.getIsRight()).count();
                    if (falseNum == 0) {//全部答对了
                        userExamQuestion.setIsRight(true);
                        userExamQuestion.setActualScore(userExamQuestion.getQuestionScore());
                        return userExamQuestion.getQuestionScore();
                    } else {//有错误答案
                        return calcBlankScoreWhenFalse(userExamQuestion, canBlankOption, dtoList);
                    }
                }
            }
        }
        return 0.0;
    }

    /**
     * 当用户答案有错误时，填空题的算分方法
     *
     * @param userExamQuestion
     * @param canBlankOption
     * @param dtoList
     * @return
     */
    public Double calcBlankScoreWhenFalse(UserExamQuestion userExamQuestion, Boolean canBlankOption, List<BlankAnswerResultDTO> dtoList) {
        if (!Objects.isNull(canBlankOption) && canBlankOption) {//是否允许按空得分
            if (dtoList.stream().filter(d -> !Objects.isNull(d.getIsRight()) && d.getIsRight()).count() == 0) {//全部错
                userExamQuestion.setIsRight(false);
                userExamQuestion.setActualScore(0.0);
                return 0.0;
            }
            //有答对的，部分得分
            double actualScore = sumRightAnswerScore(dtoList);
            userExamQuestion.setIsRight(true);
            userExamQuestion.setActualScore(actualScore);
            return actualScore;
        } else {
            userExamQuestion.setIsRight(false);
            userExamQuestion.setActualScore(0.0);
            return 0.0;
        }
    }

    /**
     * 正确项求和
     *
     * @param dtoList
     * @return
     */
    public double sumRightAnswerScore(List<BlankAnswerResultDTO> dtoList) {
        return dtoList.stream().filter(d -> !Objects.isNull(d.getIsRight()) && d.getIsRight() && !Objects.isNull(d.getScore())).mapToDouble(BlankAnswerResultDTO::getScore).sum();
    }

    /**
     * 深拷贝答案list
     *
     * @param answerList
     * @return
     */
    public List<QuestionAnswer> deepCopyAnswerList(List<QuestionAnswer> answerList) {
        List<QuestionAnswer> deepList = new ArrayList<>();
        answerList.stream().forEach(o -> {
            try {
                deepList.add(o.clone());
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        return deepList;
    }

    /**
     * 严格按照顺序对比
     *
     * @param answerList
     * @param userAnswer
     * @param dtoList
     */
    public void compareByOrder(List<QuestionAnswer> answerList, QuestionAnswer userAnswer, List<BlankAnswerResultDTO> dtoList) {
        QuestionAnswer qa = answerList.stream().filter(a -> a.getTag().equals(userAnswer.getTag())).findFirst().orElse(null);
        if (!Objects.isNull(qa)) {
            String[] answerArray = qa.getContent().trim().split("\\|\\|");
            if (!Objects.isNull(answerArray) && answerArray.length > 0) {
                List<String> asList = Arrays.asList(answerArray);
                if (asList.contains(userAnswer.getContent().trim())) {
                    dtoList.add(new BlankAnswerResultDTO(userAnswer.getTag(), userAnswer.getContent().trim(), true,
                            !Objects.isNull(qa.getPathScore()) ? qa.getPathScore().doubleValue() : null));
                } else {
                    dtoList.add(new BlankAnswerResultDTO(userAnswer.getTag(), userAnswer.getContent().trim(), false, null));
                }
            }
        }
    }

    /**
     * 不按顺序对比，只要答案在其中一个选项中找到符合的就行
     *
     * @param answerList
     * @param userAnswer
     * @param dtoList
     */
    public void compareByNoOrder(List<QuestionAnswer> answerList, QuestionAnswer userAnswer, List<BlankAnswerResultDTO> dtoList) {
        boolean isTrue = false;
        Iterator<QuestionAnswer> iterator = answerList.iterator();
        while (iterator.hasNext()) {
            QuestionAnswer qa = iterator.next();
            String[] answerArray = qa.getContent().trim().split("\\|\\|");
            if (!Objects.isNull(answerArray) && answerArray.length > 0) {
                List<String> asList = Arrays.asList(answerArray);
                if (asList.contains(userAnswer.getContent().trim())) {
                    dtoList.add(new BlankAnswerResultDTO(userAnswer.getTag(), userAnswer.getContent().trim(), true, !Objects.isNull(qa.getPathScore()) ? qa.getPathScore().doubleValue() : null));
                    isTrue = true;
                    iterator.remove();
                    break;
                }
            }
        }
        if (!isTrue) {
            dtoList.add(new BlankAnswerResultDTO(userAnswer.getTag(), userAnswer.getContent().trim(), false, null));
        }
    }

    /**
     * 多选题，设置题目得分
     *
     * @param userExamQuestion
     * @param answers
     * @param question
     * @return
     */
    public Double compareMultiToScore(UserExamQuestion userExamQuestion, String[] answers, Question question) {
        Result<Double> result = compareMultiAnswer(userExamQuestion, answers, question);
        if (Objects.isNull(result)) {
            return null;
        }
        userExamQuestion.setIsRight(result.isSuccess());
        userExamQuestion.setActualScore(result.getResult());
        return result.getResult();
    }

    /**
     * 找出正确答案
     *
     * @param answerList
     * @return
     */
    @Override
    public List<QuestionAnswer> findRightList(List<QuestionAnswer> answerList) {
        return answerList.stream().filter(o -> !Objects.isNull(o.getIsRight()) && o.getIsRight()).collect(Collectors.toList());
    }

    /**
     * 多选题，答案对比计算得分
     *
     * @param answers
     * @return
     */
    public Result<Double> compareMultiAnswer(UserExamQuestion userExamQuestion, String[] answers, Question question) {
        List<QuestionAnswer> answerList = question.getAnswerList();
        if (!Objects.isNull(answers) && answers.length > 0 && !CollectionUtils.isEmpty(answerList)) {
            //找出正确答案
            List<QuestionAnswer> rightAnswers = findRightList(answerList);
            List<String> rightAnswerIds = rightAnswers.stream().map(QuestionAnswer::getId).collect(Collectors.toList());
            List<String> userAnswerList = Arrays.asList(answers);

            //如果正确答案的个数和用户答案的个数一样，则取差集
            if (rightAnswerIds.size() == userAnswerList.size()) {
                //两个集合取差集
                //假如正确答案ABC，用户答案BCD，取差集为A
                List<String> reduceList = rightAnswerIds.stream().filter(item -> !userAnswerList.contains(item)).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(reduceList)) {
                    return Result.ok(userExamQuestion.getQuestionScore());//给整题的分
                } else {
                    return Result.error("答题错误", 0.0);//有错误项，给0分
                }
            } else {
                //去重并集，取交集，再取差集
                //假如正确答案ABCD，用户答案BCD，先取去重并集ABCD，然后取交集为BCD，然后去重并集和交集取差集为A，A为正确答案之一，给BCD的分
                //假如正确答案ABCD，用户答案BCE，先取去重并集ABCDE，然后取交集为BC，然后去重并集和交集取差集为ADE，DE为错误答案，给0分
                //假如正确答案ABCD，用户答案BCDEF，先取去重并集ABCDEF，然后取交集为BCD，然后去重并集和交集取差集为AEF，EF为错误答案，给0分
                List<String> reduceList = getReduceList(rightAnswerIds, userAnswerList);
                if (!CollectionUtils.isEmpty(reduceList)) {
                    List<QuestionAnswer> reduceAnswers = answerList.stream().filter(o -> reduceList.contains(o.getId())).collect(Collectors.toList());
                    //如果用户给的答案中有错误答案的话，给0分
                    List<QuestionAnswer> falseList = reduceAnswers.stream().filter(o -> !Objects.isNull(o.getIsRight()) && !o.getIsRight()).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(falseList)) {
                        return Result.error("答题错误", 0.0);
                    } else {
                        //漏选项单独算分
                        List<QuestionAnswer> trueList = answerList.stream().filter(o -> userAnswerList.contains(o.getId())).collect(Collectors.toList());
                        double sum = trueList.stream().filter(q -> !Objects.isNull(q.getPathScore()) && q.getPathScore() > 0).mapToDouble(QuestionAnswer::getPathScore).sum();
                        return Result.ok(sum);
                    }
                }
            }
        }
        return null;
    }

    /**
     * 获取正确答案和错误答案的差异项
     *
     * @param rightAnswerIds
     * @param userAnswerList
     * @return
     */
    public List<String> getReduceList(List<String> rightAnswerIds, List<String> userAnswerList) {
        // 并集
        List<String> listAll = rightAnswerIds.parallelStream().collect(toList());
        List<String> listAll2 = userAnswerList.parallelStream().collect(toList());
        listAll.addAll(listAll2);

        // 去重并集
        List<String> listAllDistinct = listAll.stream().distinct().collect(toList());

        // 交集
        List<String> intersection = rightAnswerIds.stream().filter(item -> userAnswerList.contains(item)).collect(toList());

        // 差集 (listAllDistinct - intersection)
        return listAllDistinct.stream().filter(item -> !intersection.contains(item)).collect(toList());
    }

    /**
     * 和答案对比算分
     *
     * @param userExamQuestion
     * @param answers
     * @param question
     * @return
     */
    public Double compareToScore(UserExamQuestion userExamQuestion, String[] answers, Question question) {
        boolean succeed = compareAnswer(answers, question);
        if (succeed) {//回答正确
            userExamQuestion.setIsRight(true);
            userExamQuestion.setActualScore(userExamQuestion.getQuestionScore());
            return userExamQuestion.getQuestionScore();
        } else {
            userExamQuestion.setIsRight(false);
            userExamQuestion.setActualScore(0.0);
            return 0.0;
        }
    }

    /**
     * 判断是否有主观题
     *
     * @param userExam
     * @return
     */
    public boolean hasSubjective(UserExam userExam) {
        return !Objects.isNull(userExam.getHasSubjective()) && userExam.getHasSubjective() ? true : false;
    }
    /**
     * 判断试卷是否有主观题
     *
     * @param userExamPapers
     * @return
     */
    public boolean hasSubjectivePaper(UserExamPapers userExamPapers) {
        return !Objects.isNull(userExamPapers.getHasSubjective()) && userExamPapers.getHasSubjective() ? true : false;
    }
    /**
     * 更新用户考试信息和成绩
     *
     * @param score
     */
    public String updateUserExamPaperAndResult(double score, UserExam userExam) {
        //四舍五入保留一位小数
        score = NumberUtil.round(score, 1).doubleValue();
        //获取当前试卷详情
        UserExamPapers userExamPapers=iUserExamPapersService.UserExamIdAllUserExamPapers(userExam.getId()).get(0);
        //组装试卷详情表
        //-----------------------------
        userExamPapers.setUserScore(score);//最终得分
        userExamPapers.setObjectiveScore(score);//客观题得分
        userExamPapers.setCommitTime(new Date());
        userExamPapers.setState(hasSubjectivePaper(userExamPapers) ? PaperState.WAIT_OPT : PaperState.FINISHED);
        //计算考试时长
        int userpaperTime = (int) ((System.currentTimeMillis() - userExamPapers.getCreateTime().getTime()) / 1000 / 60);
        if (userpaperTime == 0) {
            userpaperTime = 1;
        }

        //更新考试记录
        userExamPapers.setUserTime(userpaperTime);
        int paperpass = score >= userExamPapers.getQualifyScore() ? 1 : 0;
        userExamPapers.setPassed(paperpass);
        //-----------------------------
        //判断考试试卷是否全部做完，未全部做完只更新试卷状态，全部考完更新考试状态及试卷状态
        //获取考试绑定了几套试卷
        List<ExamPapers> examPapersList= iExamPapersService.ExamIdAllExamPapers(userExam.getExamId());
        //判断当前试卷是不是最后一套试卷
        if(userExam.getPaperId().equals(examPapersList.get(examPapersList.size()-1).getPaperId())){
            //组装考试记录表
            userExam.setCommitTime(new Date());
            userExam.setState(hasSubjective(userExam) ? PaperState.WAIT_OPT : PaperState.FINISHED);
            //考试完成后加入成绩
//            userExamResultService.joinPaperResult(userExam.getUserId(), userExam.getExamId(),userExam.getPaperId(), score, paperpass);
            //更新试卷状态
            iUserExamPapersService.updateById(userExamPapers);
            //更新考试状态
            userExamService.updateById(userExam);
            return "0";
        }else {//只更新试卷状态
            //考试完成后加入成绩
//            userExamResultService.joinPaperResult(userExam.getUserId(), userExam.getExamId(),userExam.getPaperId(), score, paperpass);
            //更新试卷状态
            iUserExamPapersService.updateById(userExamPapers);
            return "1";
        }
//        userExam.setUserScore(score);//最终得分
//        userExam.setObjectiveScore(score);//客观题得分
//        userExam.setCommitTime(new Date());
//        userExam.setState(hasSubjective(userExam) ? PaperState.WAIT_OPT : PaperState.FINISHED);
//        //计算考试时长
//        int userTime = (int) ((System.currentTimeMillis() - userExam.getCreateTime().getTime()) / 1000 / 60);
//        if (userTime == 0) {
//            userTime = 1;
//        }
//
//        //更新考试记录
//        userExam.setUserTime(userTime);
//        int pass = score >= userExam.getQualifyScore() ? 1 : 0;
//        userExam.setPassed(pass);
    }
    /**
     * 更新用户考试信息和成绩
     *
     * @param score
     */
    public void updateUserExamAndResult(double score, UserExam userExam) {
        //四舍五入保留一位小数
        score = NumberUtil.round(score, 1).doubleValue();
        userExam.setUserScore(score);//最终得分
        userExam.setObjectiveScore(score);//客观题得分
        userExam.setCommitTime(new Date());
        userExam.setState(hasSubjective(userExam) ? PaperState.WAIT_OPT : PaperState.FINISHED);
        //计算考试时长
        int userTime = (int) ((System.currentTimeMillis() - userExam.getCreateTime().getTime()) / 1000 / 60);
        if (userTime == 0) {
            userTime = 1;
        }

        //更新考试记录
        userExam.setUserTime(userTime);
        int pass = score >= userExam.getQualifyScore() ? 1 : 0;
        userExam.setPassed(pass);

        //考试完成后加入成绩
        userExamResultService.joinResult(userExam.getUserId(), userExam.getExamId(), score, pass);

        userExamService.updateById(userExam);
    }

    /**
     * 提交的答案和题目答案对比
     *
     * @param answers
     * @return
     */
    public boolean compareAnswer(String[] answers, Question question) {
        List<QuestionAnswer> answerList = question.getAnswerList();
        if (!Objects.isNull(answers) && answers.length > 0 && !CollectionUtils.isEmpty(answerList)) {
            //找出正确答案
            List<QuestionAnswer> rightAnswers = findRightList(answerList);
            List<String> rightAnswerIds = rightAnswers.stream().map(QuestionAnswer::getId).collect(Collectors.toList());
            List<String> userAnswerList = Arrays.asList(answers);

            //如果正确答案的个数和用户答案的个数一样，则取差集
            if (rightAnswerIds.size() == userAnswerList.size()) {
                //两个集合取差集
                List<String> reduceList = rightAnswerIds.stream().filter(item -> !userAnswerList.contains(item)).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(reduceList)) {
                    return true;
                }
                return false;
            }
            return false;
        }
        return false;
    }


    /**
     * 缓存考试答案，用于定时任务强制交卷
     *
     * @param dto
     */
    @Override
    public void cacheExamAnswer(ExamSubmitDTO dto) {
        if (StringUtils.isNotBlank(dto.getUserExamId()) && !Objects.isNull(dto.getLimitTime())) {
            //缓存答案的时间为截止时间后3天
            long expireMins = DateUtil.between(new Date(), dto.getLimitTime(), DateUnit.MINUTE) + 60 * 24 * 3;
            long expireSec = expireMins * 60;
            redisUtil.set(dto.getUserExamId(), JSON.toJSONString(dto), expireSec);
        }
    }


    /**
     * 缓存考试试卷答案，用于定时任务强制交卷
     *
     * @param dto
     */
    @Override
    public void cacheExamPaperAnswer(ExamSubmitDTO dto) {
        if (StringUtils.isNotBlank(dto.getUserExamId()) && !Objects.isNull(dto.getLimitTime())) {
            //缓存答案的时间为截止时间后3天
            long expireMins = DateUtil.between(new Date(), dto.getLimitTime(), DateUnit.MINUTE) + 60 * 24 * 3;
            long expireSec = expireMins * 60;
            String idkey=dto.getUserExamId()+dto.getPaperId();
            System.out.println("-----------------------------");
            System.out.println(JSON.toJSONString(dto));
            System.out.println("-----------------------------");
            redisUtil.set(idkey, JSON.toJSONString(dto), expireSec);
        }
    }


    /**
     * 获取正在进行中的考试
     *
     * @return
     */
    @Override
    public List<String> listExamIng() {
        return examMapper.listExamIng();
    }


    /**
     * 删除自动交卷任务
     *
     * @param parameter
     */
    @Override
    public void deleteAutoCommitJob(String parameter) {
        //删除定时任务
        List<QuartzJob> jobList = quartzJobService.findByCondition(BreakExamJob.class.getName(), parameter);
        if (!CollectionUtils.isEmpty(jobList)) {
            jobList.stream().forEach(quartzJob -> quartzJobService.deleteAndStopJob(quartzJob));
        }
    }

    /**
     * 检查试卷是否已经被用于考试
     *
     * @param paperId
     * @return
     */
    @Override
    public boolean checkIsUsedInExam(String paperId) {
        LambdaQueryWrapper<Exam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Exam::getPaperId, paperId);
        return this.count(wrapper) > 0 ? true : false;
    }

    /**
     * 获取缓存的考试答案
     *
     * @return
     */
    @Override
    public ExamSubmitDTO getCacheAnswer(String userExamId) {
        Object answerObject = redisUtil.get(userExamId);
        if (!Objects.isNull(answerObject)) {
            return JSON.parseObject(JSON.parse(String.valueOf(answerObject)).toString(), ExamSubmitDTO.class);
        }
        return null;
    }
    /**
     * 获取缓存的考试试卷答案
     *
     * @return
     */
    @Override
    public ExamSubmitDTO getCacheExamPaperAnswer(String userExamId) {
        //获取考试中的试卷
        String idkey=userExamId+iUserExamPapersService.UserExamIdAllUserExamPapers(userExamId).get(0).getPaperId();
        Object answerObject = redisUtil.get(idkey);
//        System.out.println("yyz显示显示redisUtil.get："+answerObject);
        if (!Objects.isNull(answerObject)) {
            return JSON.parseObject(JSON.parse(String.valueOf(answerObject)).toString(), ExamSubmitDTO.class);
        }
        return null;
    }
    /**
     * 清除缓存的考试答案
     *
     * @return
     */
    @Override
    public void deleteCacheAnswer(String userExamId) {
        redisUtil.del(userExamId);
    }

    /**
     * 检查是否达到考试限制次数
     *
     * @param userId
     * @param examId
     * @return
     */
    @Override
    public Boolean checkToLimit(String userId, String examId) {
        Exam exam = getById(examId);
        if (!Objects.isNull(exam.getLimitCount())) {
            long userExamNum = userExamService.countUserExam(userId, examId);
            if (userExamNum >= exam.getLimitCount()) { //考试次数达到上限
                return true;
            }
        }
        return false;
    }

    /**
     * 检查是否有考试记录
     *
     * @param examId
     * @return
     */
    @Override
    public Boolean examRecordExist(String userId, String examId) {
        long userExamNum = userExamService.countUserExam(userId, examId);
        return userExamNum > 0;
    }

    /**
     * 最新考试
     *
     * @return
     */
    @Override
    public List<Exam> listLatestExam(Integer limit, String userId) {
        return examMapper.listLatestExam(limit, userId);
    }


    /**
     * 获取考试状态
     *
     * @param exam
     * @return
     */
    @Override
    public Integer getExamState(Exam exam) {
        if (!Objects.isNull(exam.getStartTime())) {
            if (System.currentTimeMillis() < exam.getStartTime().getTime()) {
                return ExamState.READY_START;
            }
        }
        if (!Objects.isNull(exam.getEndTime())) {
            if (System.currentTimeMillis() > exam.getEndTime().getTime()) {
                return ExamState.OVERDUE;
            }
        }
        if (!Objects.isNull(exam.getStartTime()) && !Objects.isNull(exam.getEndTime())) {
            if (System.currentTimeMillis() > exam.getStartTime().getTime()
                    && System.currentTimeMillis() < exam.getEndTime().getTime()) {
                return ExamState.ENABLE;
            }
        }
        return null;
    }
}
