package com.ynfy.buss.exam.examstatistics.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.exam.examstatistics.entity.ExamStatistics;
import com.ynfy.buss.exam.examstatistics.service.IExamStatisticsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 考试统计
 * @Author: jeecg-boot
 * @Date: 2023-03-20
 * @Version: V1.0
 */
@Api(tags = "考试统计")
@RestController
@RequestMapping("/examStatistics")
@Slf4j
public class ExamStatisticsController extends JeecgController<ExamStatistics, IExamStatisticsService> {
    @Autowired
    private IExamStatisticsService examStatisticsService;

    /**
     * 分页列表查询
     *
     * @param examStatistics
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "考试统计-分页列表查询", notes = "考试统计-分页列表查询")
    @RequiresPermissions("examStatistics:list")
    @GetMapping(value = "/list")
    public Result<IPage<ExamStatistics>> queryPageList(ExamStatistics examStatistics,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        QueryWrapper<ExamStatistics> queryWrapper = QueryGenerator.initQueryWrapper(examStatistics, req.getParameterMap());
        Page<ExamStatistics> page = new Page<ExamStatistics>(pageNo, pageSize);
        IPage<ExamStatistics> pageList = examStatisticsService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param examStatistics
     * @return
     */
    @AutoLog(value = "考试统计-添加")
    @ApiOperation(value = "考试统计-添加", notes = "考试统计-添加")
    //@RequiresPermissions("examStatistics:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody ExamStatistics examStatistics) {
        examStatisticsService.save(examStatistics);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param examStatistics
     * @return
     */
    @AutoLog(value = "考试统计-编辑")
    @ApiOperation(value = "考试统计-编辑", notes = "考试统计-编辑")
    //@RequiresPermissions("examStatistics:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody ExamStatistics examStatistics) {
        examStatisticsService.updateById(examStatistics);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "考试统计-通过id删除")
    @ApiOperation(value = "考试统计-通过id删除", notes = "考试统计-通过id删除")
    //@RequiresPermissions("examStatistics:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        examStatisticsService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "考试统计-批量删除")
    @ApiOperation(value = "考试统计-批量删除", notes = "考试统计-批量删除")
    //@RequiresPermissions("examStatistics:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.examStatisticsService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "考试统计-通过id查询")
    @ApiOperation(value = "考试统计-通过id查询", notes = "考试统计-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ExamStatistics> queryById(@RequestParam(name = "id", required = true) String id) {
        ExamStatistics examStatistics = examStatisticsService.getById(id);
        if (examStatistics == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(examStatistics);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param examStatistics
     */
    //@RequiresPermissions("examStatistics:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, ExamStatistics examStatistics) {
        return super.exportXls(request, examStatistics, ExamStatistics.class, "考试统计表");
    }

    /**
     * 考试结果统计 定时任务
     *
     * @param examId
     * @return
     */
    @ApiOperation(value = "考试统计-考试结果统计", notes = "考试统计-考试结果统计")
    @GetMapping(value = "/calcExamDimension")
    public Result<?> calcExamDimension(String examId) {
        return Result.OK(examStatisticsService.calcExamDimension(examId));
    }

}
