package com.ynfy.buss.exam.userexamquestionanswer.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.exam.userexamquestionanswer.entity.UserExamQuestionAnswer;
import com.ynfy.buss.exam.userexamquestionanswer.service.IUserExamQuestionAnswerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: 用户考试答案
 * @Author: jeecg-boot
 * @Date: 2023-04-01
 * @Version: V1.0
 */
@Api(tags = "用户考试答案")
@RestController
@RequestMapping("/serExamQuestionAnswer")
@Slf4j
public class UserExamQuestionAnswerController extends JeecgController<UserExamQuestionAnswer, IUserExamQuestionAnswerService> {
    @Autowired
    private IUserExamQuestionAnswerService userExamQuestionAnswerService;

    /**
     * 分页列表查询
     *
     * @param userExamQuestionAnswer
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "用户考试答案-分页列表查询")
    @ApiOperation(value = "用户考试答案-分页列表查询", notes = "用户考试答案-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserExamQuestionAnswer>> queryPageList(UserExamQuestionAnswer userExamQuestionAnswer,
                                                               @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                               @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                               HttpServletRequest req) {
        QueryWrapper<UserExamQuestionAnswer> queryWrapper = QueryGenerator.initQueryWrapper(userExamQuestionAnswer, req.getParameterMap());
        Page<UserExamQuestionAnswer> page = new Page<UserExamQuestionAnswer>(pageNo, pageSize);
        IPage<UserExamQuestionAnswer> pageList = userExamQuestionAnswerService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param userExamQuestionAnswer
     * @return
     */
    @AutoLog(value = "用户考试答案-添加")
    @ApiOperation(value = "用户考试答案-添加", notes = "用户考试答案-添加")
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody UserExamQuestionAnswer userExamQuestionAnswer) {
        userExamQuestionAnswerService.save(userExamQuestionAnswer);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param userExamQuestionAnswer
     * @return
     */
    @AutoLog(value = "用户考试答案-编辑")
    @ApiOperation(value = "用户考试答案-编辑", notes = "用户考试答案-编辑")
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody UserExamQuestionAnswer userExamQuestionAnswer) {
        userExamQuestionAnswerService.updateById(userExamQuestionAnswer);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户考试答案-通过id删除")
    @ApiOperation(value = "用户考试答案-通过id删除", notes = "用户考试答案-通过id删除")
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        userExamQuestionAnswerService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户考试答案-批量删除")
    @ApiOperation(value = "用户考试答案-批量删除", notes = "用户考试答案-批量删除")
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.userExamQuestionAnswerService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "用户考试答案-通过id查询")
    @ApiOperation(value = "用户考试答案-通过id查询", notes = "用户考试答案-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserExamQuestionAnswer> queryById(@RequestParam(name = "id", required = true) String id) {
        UserExamQuestionAnswer userExamQuestionAnswer = userExamQuestionAnswerService.getById(id);
        if (userExamQuestionAnswer == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(userExamQuestionAnswer);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param userExamQuestionAnswer
     */
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, UserExamQuestionAnswer userExamQuestionAnswer) {
        return super.exportXls(request, userExamQuestionAnswer, UserExamQuestionAnswer.class, "用户考试答案");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("exam.userexamquestionanswer:user_exam_question_answer:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, UserExamQuestionAnswer.class);
    }

}
