package com.ynfy.buss.practice.userpracticequestion.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.practice.userpracticequestion.entity.UserPracticeQuestion;
import com.ynfy.buss.practice.userpracticequestion.service.IUserPracticeQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: 用户练习题目表
 * @Author: jeecg-boot
 * @Date: 2023-11-19
 * @Version: V1.0
 */
@Api(tags = "用户练习题目表")
@RestController
@RequestMapping("/practice/userPracticeQuestion")
@Slf4j
public class UserPracticeQuestionController extends JeecgController<UserPracticeQuestion, IUserPracticeQuestionService> {
    @Autowired
    private IUserPracticeQuestionService userPracticeQuestionService;

    /**
     * 分页列表查询
     *
     * @param userPracticeQuestion
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "用户练习题目表-分页列表查询")
    @ApiOperation(value = "用户练习题目表-分页列表查询", notes = "用户练习题目表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserPracticeQuestion>> queryPageList(UserPracticeQuestion userPracticeQuestion,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             HttpServletRequest req) {
        QueryWrapper<UserPracticeQuestion> queryWrapper = QueryGenerator.initQueryWrapper(userPracticeQuestion, req.getParameterMap());
        Page<UserPracticeQuestion> page = new Page<UserPracticeQuestion>(pageNo, pageSize);
        IPage<UserPracticeQuestion> pageList = userPracticeQuestionService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param userPracticeQuestion
     * @return
     */
    @AutoLog(value = "用户练习题目表-添加")
    @ApiOperation(value = "用户练习题目表-添加", notes = "用户练习题目表-添加")
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody UserPracticeQuestion userPracticeQuestion) {
        userPracticeQuestionService.save(userPracticeQuestion);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param userPracticeQuestion
     * @return
     */
    @AutoLog(value = "用户练习题目表-编辑")
    @ApiOperation(value = "用户练习题目表-编辑", notes = "用户练习题目表-编辑")
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody UserPracticeQuestion userPracticeQuestion) {
        userPracticeQuestionService.updateById(userPracticeQuestion);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户练习题目表-通过id删除")
    @ApiOperation(value = "用户练习题目表-通过id删除", notes = "用户练习题目表-通过id删除")
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        userPracticeQuestionService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户练习题目表-批量删除")
    @ApiOperation(value = "用户练习题目表-批量删除", notes = "用户练习题目表-批量删除")
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.userPracticeQuestionService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "用户练习题目表-通过id查询")
    @ApiOperation(value = "用户练习题目表-通过id查询", notes = "用户练习题目表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserPracticeQuestion> queryById(@RequestParam(name = "id", required = true) String id) {
        UserPracticeQuestion userPracticeQuestion = userPracticeQuestionService.getById(id);
        if (userPracticeQuestion == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(userPracticeQuestion);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param userPracticeQuestion
     */
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, UserPracticeQuestion userPracticeQuestion) {
        return super.exportXls(request, userPracticeQuestion, UserPracticeQuestion.class, "用户练习题目表");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("practice.userpracticequestion:user_practice_question:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, UserPracticeQuestion.class);
    }

}
